/*
 * CodeAidOptionPane.java - Panel in jEdit's Global Options dialog
 * Copyright (C) 1999 Jason Ginchereau
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


import javax.swing.JCheckBox;
import javax.swing.JTextField;

import org.gjt.sp.jedit.AbstractOptionPane;
import org.gjt.sp.jedit.jEdit;

import org.gjt.sp.util.Log;


public class CodeAidOptionPane extends AbstractOptionPane {
    private JCheckBox  showSyntaxErrorsCB;
    private JCheckBox  showSemanticErrorsCB;
    private JCheckBox  membersCB;
    private JTextField membersDelayTF;
    private JCheckBox  methodsCB;
    private JTextField methodsDelayTF;


    public CodeAidOptionPane() {
        super("codeaid");
    }


    public void _init() {
        this.showSyntaxErrorsCB   =
            this.createCheckBox("codeaid.show-syntax-errors", false);
        this.showSemanticErrorsCB =
            this.createCheckBox("codeaid.show-semantic-errors", false);
        this.membersCB =
            this.createCheckBox("codeaid.automembers", true);
        this.membersDelayTF =
            this.createTextField(4, "codeaid.automembers-delay", "500");
        this.methodsCB =
            this.createCheckBox("codeaid.automethods", true);
        this.methodsDelayTF =
            this.createTextField(4, "codeaid.automethods-delay", "500");

        addComponent(this.membersCB);
        addComponent(jEdit.getProperty("options.codeaid.automembers-delay"),
            this.membersDelayTF);
        addComponent(this.methodsCB);
        addComponent(jEdit.getProperty("options.codeaid.automethods-delay"),
            this.methodsDelayTF);
        addComponent(this.showSyntaxErrorsCB);
        addComponent(this.showSemanticErrorsCB);
    }


    /**
     * Called when the options dialog's `OK' button is pressed.
     * This should save any properties saved in this option pane.
    **/
    public void _save() {
        jEdit.setBooleanProperty("codeaid.show-syntax-errors",
            this.showSyntaxErrorsCB.isSelected()
        );
        jEdit.setBooleanProperty("codeaid.show-semantic-errors",
            this.showSemanticErrorsCB.isSelected()
        );
        jEdit.setBooleanProperty("codeaid.automembers",
            this.membersCB.isSelected()
        );
        jEdit.setProperty("codeaid.automembers-delay",
            "" + CodeAidUtilities.getInteger(this.membersDelayTF.getText(), 500)
        );
        jEdit.setBooleanProperty("codeaid.automethods",
            this.methodsCB.isSelected()
        );
        jEdit.setProperty("codeaid.automethods-delay",
            "" + CodeAidUtilities.getInteger(this.methodsDelayTF.getText(), 500)
        );
    }


    private JCheckBox createCheckBox(String property, boolean defaultValue) {
        JCheckBox cb = new JCheckBox(jEdit.getProperty("options." + property));
        cb.setSelected(jEdit.getBooleanProperty(property, defaultValue));
        return cb;
    }


    private JTextField createTextField(int size, String property, String defaultValue) {
        JTextField tf = new JTextField(jEdit.getProperty(property, defaultValue), size);
        return tf;
    }
}
