/*
 * CodeAidPlugin.java - main class of the CodeAid plugin for jEdit
 * Copyright (C) 1999 Jason Ginchereau
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


import org.gjt.sp.jedit.EditPlugin;
import org.gjt.sp.jedit.GUIUtilities;
import org.gjt.sp.jedit.gui.OptionsDialog;

import java.util.Enumeration;
import java.util.Vector;

// Collections API
import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedList;


/**
 * This class just creates and delegates to all the different parts
 * of the CodeAid plugin.
 *
 * @author Jason Ginchereau
 * @author Andre Kaplan
 */
public class CodeAidPlugin extends EditPlugin {
    private Collection pluginParts;


    public CodeAidPlugin() {
        this.pluginParts = new LinkedList();
        this.pluginParts.add(new codeaid.info.CodeAidInfoPluginPart());
        this.pluginParts.add(new codeaid.parser.CodeAidParserPluginPart());
        this.pluginParts.add(new codeaid.errors.CodeAidErrorsPluginPart());
        this.pluginParts.add(new codeaid.popup.CodeAidPopupPluginPart());
        this.pluginParts.add(new codeaid.browser.CodeAidBrowserPluginPart());
    }


    public void start() {
        for (Iterator i = this.pluginParts.iterator(); i.hasNext(); ) {
            ((EditPlugin) i.next()).start();
        }
    }


    public void stop() {
        for (Iterator i = this.pluginParts.iterator(); i.hasNext(); ) {
            ((EditPlugin) i.next()).stop();
        }
    }


    public void createMenuItems(Vector menuItems) {
        menuItems.addElement(GUIUtilities.loadMenu("codeaid-menu"));
    }


    public void createOptionPanes(OptionsDialog od) {
        od.addOptionPane(new CodeAidOptionPane());
    }
}
