/*
 * CodeAidPopupPlugin.java - part of the CodeAid plugin for jEdit
 * Copyright (C) 1999 Jason Ginchereau
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


package codeaid.browser;

import codeaid.info.ClassInfo;
import codeaid.info.MemberInfo;
import codeaid.info.ClassTable;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.IOException;

// Collections API
import java.util.Iterator;

import java.util.zip.ZipFile;
import javax.swing.*;


/**
 *
 * @author Jason Ginchereau
**/
public class ClassBrowser extends JPanel {
    private ClassTable classTable;
    private JTextField classNameField;
    private JTextArea  classInfoArea;


    public static void main(String[] args) {
        if (args.length != 1) {
            System.err.println("Usage: cdbbrowse <filename.cdb>");
            return;
        }

        ZipFile cdb;
        try {
            cdb = new ZipFile(args[0]);
        } catch(IOException ioex) {
            System.err.println(ioex);
            return;
        }

        ClassTable ct = new ClassTable();
        ct.preLoad(cdb);

        JFrame f = new JFrame("CodeAid Class Browser");
        f.getContentPane().add(new ClassBrowser(ct));
        f.setSize(640, 480);
        f.setVisible(true);
        f.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent evt) {
                System.exit(0);
            }
        });
    }


    public ClassBrowser() {
        this(ClassTable.getInstance());
    }


    public ClassBrowser(ClassTable classTable) {
        this.classTable = classTable;

        Font f = new Font("Monospaced", Font.PLAIN, 12);

        classNameField = new JTextField();
        classNameField.setFont(f);
        classInfoArea = new JTextArea();
        classInfoArea.setFont(f);

        setLayout(new BorderLayout());
        add(BorderLayout.NORTH, classNameField);
        add(BorderLayout.CENTER, new JScrollPane(classInfoArea));

        classNameField.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                updateInfo();
            }
        });
    }


    private void updateInfo() {
        String className = classNameField.getText();
        ClassInfo ci = classTable.get(className);
        if (ci == null) {
            classInfoArea.setText("");
        } else {
            classInfoArea.setText(getDebugString(ci));
        }
    }


    private static String getDebugString(ClassInfo ci) {
        String s = "" + ci.getComment() + '\n';
        s += ci.getLine();
        String[] ifs = ci.getInterfaces();
        if (ci.isInterface()) {
            if (ifs.length > 0) {
                s += " extends ";
                for (int i = 0; i < ifs.length; i++) {
                    s += ifs[i];
                    if (i < ifs.length-1) { s += ", "; }
                }
            }
        } else {
            s += " extends " + ci.getSuperclass();
            if (ifs.length > 0) {
                s += " implements ";
                for (int i = 0; i < ifs.length; i++) {
                    s += ifs[i];
                    if (i < ifs.length-1) { s += ", "; }
                }
            }
        }
        s += " {" + '\n';

        for (Iterator i = ci.getFields().iterator(); i.hasNext(); ) {
            s += "  " + ((MemberInfo)i.next()).getLine() + '\n';
        }

        for (Iterator i = ci.getConstructors().iterator(); i.hasNext(); ) {
            s += "  " + ((MemberInfo)i.next()).getLine() + '\n';
        }

        for(Iterator i = ci.getMethods().iterator(); i.hasNext(); ) {
            s += "  " + ((MemberInfo)i.next()).getLine() + '\n';
        }

        for(Iterator i = ci.getClasses().iterator(); i.hasNext(); ) {
            s += "  class " + i.next() + '\n';
        }

        s += "}" + '\n';
        return s;
    }
}
