/*
 * CodeAidInfoPluginPart.java - part of the CodeAid plugin for jEdit
 * Copyright (C) 1999 Jason Ginchereau
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


package codeaid.info;

import org.gjt.sp.jedit.EditPlugin;
import org.gjt.sp.jedit.jEdit;
import org.gjt.sp.util.Log;

import java.io.IOException;
import java.io.File;

import java.util.Enumeration;
import java.util.Hashtable;

import java.util.zip.ZipFile;


/**
 *
 * @author Jason Ginchereau
**/
public class CodeAidInfoPluginPart extends EditPlugin {
    private static Hashtable cdbFiles = new Hashtable();


    public void start() {
        File cdbDir = new File(jEdit.getSettingsDirectory() + File.separator
            + "codeaid");
        if(!cdbDir.exists()) {
            cdbDir.mkdir();
        } else {
            ClassTable ct = ClassTable.getInstance();
            ZipFile cdb;
            String[] fileList = cdbDir.list();
            for(int i = 0; i < fileList.length; i++) {
                if(!fileList[i].endsWith(".cdb")) continue;
                try {
                    cdb = new ZipFile(new File(cdbDir, fileList[i]));
                } catch(IOException ioex) {
                    Log.log(Log.WARNING, this, "Invalid .cdb file: [" + fileList[i] + "]");
                    cdb = null;
                }
                if(cdb != null) {
                    ct.preLoad(cdb);
                    Log.log(Log.MESSAGE, this,
                        "CodeAid DB loaded successfully: [" + fileList[i] + "]");
                    cdbFiles.put(fileList[i], cdb);
                }
            }
        }
    }


    public void stop() {
        ClassTable ct = ClassTable.getInstance();
        for (Enumeration e = cdbFiles.keys(); e.hasMoreElements(); ) {
            String fileName = (String)e.nextElement();
            Log.log(Log.MESSAGE, this, "Unloading: [" + fileName + "]");
            ZipFile cdb = (ZipFile) cdbFiles.get(fileName);
            cdbFiles.remove(fileName);
            ct.unload(cdb);
            try {
                cdb.close();
            } catch (IOException ioe) {
                Log.log(Log.ERROR, this, ioe);
            }
        }
    }


    public static ZipFile getCDBFile(String name) {
        return (ZipFile)cdbFiles.get(name);
    }


    public static ZipFile removeCDBFile(String name) {
        return (ZipFile)cdbFiles.remove(name);
    }
}
