/*
 * CodeAidParserPluginPart.java - part of the CodeAid plugin for jEdit
 * Copyright (C) 1999 Jason Ginchereau
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


package codeaid.parser;

import org.gjt.sp.jedit.Buffer;
import org.gjt.sp.jedit.EBMessage;
import org.gjt.sp.jedit.EBPlugin;
import org.gjt.sp.jedit.EditBus;
import org.gjt.sp.jedit.jEdit;
import org.gjt.sp.jedit.msg.BufferUpdate;
import org.gjt.sp.jedit.msg.EditPaneUpdate;
import org.gjt.sp.jedit.msg.PropertiesChanged;

import javax.swing.text.Document;

// Collections API
import java.util.Iterator;
import java.util.HashMap;
import java.util.Map;


/**
 *
 * @author Jason Ginchereau
**/
public class CodeAidParserPluginPart
    extends EBPlugin
{
    private static boolean showSyntaxErrors =
        jEdit.getBooleanProperty("codeaid.show-syntax-errors", false);

    private static boolean showSemanticErrors =
        jEdit.getBooleanProperty("codeaid.show-semantic-errors", false);

    /**
     * Maps from jEdit Buffers to JavaParsers.
    **/
    static Map parserTable = new HashMap();


    public void start() {}


    public void stop() {}


    public void handleMessage(EBMessage message)
    {
        if (message instanceof BufferUpdate) {
            BufferUpdate bu = (BufferUpdate) message;

            if (bu.getWhat() == BufferUpdate.CREATED)
            {
                Buffer b = bu.getBuffer();
                String filename = b.getName();
                if (filename.toLowerCase().endsWith(".java")) {
                    JavaParser jp = new JavaParser();
                    parserTable.put(b, jp);
                    jp.generateClassInfo = true;
                    //jp.generateExpressionInfo = true;
                    jp.generateSyntaxErrorInfo   = showSyntaxErrors;
                    jp.generateSemanticErrorInfo = showSemanticErrors;
                    jp.init(b, b.getPath());
                    jp.start();
                }
            } else if (bu.getWhat() == BufferUpdate.CLOSED) {
                JavaParser s = (JavaParser) parserTable.remove(bu.getBuffer());
                if (s != null) { s.stopSoon(); }
            }
        } else if (message instanceof PropertiesChanged) {
            propertiesChanged();
        }
    }


    /**
     * Returns the <code>JavaParser</code> that is associated with a
     * <code>Document</code>, or <code>null</code> if there is none.
    **/
    public static JavaParser getParser(Document doc) {
        return (JavaParser) parserTable.get(doc);
    }


    private static void propertiesChanged() {
        boolean newShowSyntaxErrors =
            jEdit.getBooleanProperty("codeaid.show-syntax-errors", false);

        boolean newShowSemanticErrors =
            jEdit.getBooleanProperty("codeaid.show-semantic-errors", false);

        if (   (newShowSyntaxErrors   == showSyntaxErrors)
            && (newShowSemanticErrors == showSemanticErrors)
        ) {
            return;
        }

        showSyntaxErrors   = newShowSyntaxErrors;
        showSemanticErrors = newShowSemanticErrors;

        for (Iterator i = parserTable.values().iterator(); i.hasNext(); ) {
            JavaParser parser = (JavaParser) i.next();
            parser.generateSyntaxErrorInfo   = showSyntaxErrors;
            parser.generateSemanticErrorInfo = showSemanticErrors;
            parser.setDocChange(true);
            parser.initErrorTable();
        }
    }
}
