/* Generated By:JavaCC: Do not edit this line. JavaParser.java */
package codeaid.parser;

import codeaid.info.*;

import java.io.IOException;
import java.io.File;
import java.io.Reader;
import java.io.FileReader;
import java.io.StringReader;
import java.lang.reflect.Modifier;

// Collections API
import java.util.Iterator;
import java.util.List;
import java.util.LinkedList;

import javax.swing.text.BadLocationException;
import javax.swing.text.Document;
import javax.swing.text.Segment;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

import org.gjt.sp.util.Log;


/**
 * @author Jason Ginchereau
**/
public final class JavaParser extends Thread implements DocumentListener, JavaParserConstants {
    public boolean oneLineComments           = true;
    public boolean generateClassInfo         = false;
    public boolean generatePrivateClassInfo  = true;
    public boolean generateSyntaxErrorInfo   = false;
    public boolean generateSemanticErrorInfo = false;

    public ClassTable classTable = null;
    public ErrorTable errorTable = null;

    private static final int DEFAULT_PARSER_PRIORITY = Thread.MIN_PRIORITY;

    private   Document       document;
    private   String         path;
    private   Reader         reader;
    private   boolean        docChange;

    // information gathered during parsing
    private SymbolTable     symbolTable;
    private SymbolTable     newSymbolTable;
    private ErrorTable      newErrorTable;
    private List            imports;
    private List            newImports;

    private int offsetLine = 0;
    private int offsetCol = 0;

    // temporaries for use during parsing
    private String          comment;
    private int             modifiers;
    private SourceLocation  sourceLocation;
    private SourceLocation  sourceLocation1;
    private String          packageName;
    private List            symbolPath;
    private Token           staticPrimary;


    public JavaParser() {
        this((java.io.Reader)null);

        setName("JavaParser");
        setPriority(DEFAULT_PARSER_PRIORITY);

        document = null;
        path   = null;
        reader = null;
        docChange = false;

        symbolTable = null;
        //exprTable = null;
        imports = null;
        symbolPath = new LinkedList();
    }


    public void initErrorTable() {
        if (generateSyntaxErrorInfo || generateSemanticErrorInfo) {
            if (errorTable == null) {
                errorTable = ErrorTable.getInstance();
            }
        } else {
            if (errorTable != null) {
                errorTable.replace(getPath(), new ErrorTable());
            }
        }
    }


    public void init(Reader reader, String path) {
        this.reader = reader;
        this.path   = path;

        classTable = ClassTable.getInstance();
        initErrorTable();
    }


    public void init(File file) throws IOException {
        init(new FileReader(file), file.getPath());
    }


    public void init(Document doc, String path) {
        init(new DocumentReader(doc), path);
        this.document = doc;
        docChange = true;
        document.addDocumentListener(this);
    }


    public void setDocChange(boolean docChange) {
        this.docChange = docChange;
    }


    public void run() {
        if (reader == null) {
            throw new IllegalStateException("Must call init() before run().");
        }

        if (document != null) {
            do {
                // sleep until a change is made to the document
                if (!docChange) {
                    try { synchronized (this) { wait(); } }
                    catch(InterruptedException iex) { }
                    continue;
                }
                docChange = false;

                ((DocumentReader) reader).reset();
                oneParsePass();
            } while (document != null);

            if (generateSyntaxErrorInfo || generateSemanticErrorInfo) {
                errorTable.replace(getPath(), new ErrorTable());
            }
        } else {
            oneParsePass();
        }
    }


    private void oneParsePass() {
        ReInit(reader);

        preParse();
        try {
            parseFile();
        } catch(ParseException pex) {
            if(generateSyntaxErrorInfo) {
                error(pex.currentToken.next, "Syntax error.");
            }
            //pex.printStackTrace(System.err);
        } catch(TokenMgrError tme) {
            //tme.printStackTrace(System.err);
        }
        postParse();
    }


    /**
     * Avoids using Thread.stop()
    **/
    public synchronized void stopSoon() {
        if(document != null) {
            document.removeDocumentListener(this);
            document = null;
        }
        notify();
    }


    // DocumentListener interface
    // Reparses the buffer when more than 16 characters are inserted or ; or }
    public void insertUpdate(DocumentEvent evt) {
        if (evt.getLength() >= 16) {
             docChange = true;
        } else {
            Segment segment = new Segment();
            try {
                this.document.getText(evt.getOffset(), evt.getLength(), segment);
            } catch (BadLocationException ble) {
                Log.log(Log.ERROR, this, "insertUpdate: " + ble);
            }
            int off = segment.offset;
            char[] array = segment.array;
            for (int i = segment.count - 1; i >= 0; i--, off++) {
                if (array[off] == ';' || array[off] == '}') {
                    docChange = true;
                    break;
                }
            }
        }

        synchronized (this) {
            notify();
        }
    }


    // Reparses the buffer when more than 2 characters are deleted
    public void removeUpdate(DocumentEvent evt) {
        // Log.log(Log.DEBUG, this, "removeUpdate: " + evt.getLength() + "]");
        if (evt.getLength() >= 2) {
            docChange = true;
        }

        synchronized (this) {
            notify();
        }
    }


    public void changedUpdate(DocumentEvent evt) {
        Log.log(Log.DEBUG, this, "changedUpdate: " + evt.getLength() + "]");
        docChange = true;

        synchronized (this) {
            notify();
        }
    }


    /**
     * Returns the path name of the code being parsed.
    **/
    public String getPath() {
        return path;
    }


    private void parseFile() throws ParseException {
        CompilationUnit();
    }


    private void preParse() {
        comment = null;
        modifiers = 0;
        sourceLocation = null;
        sourceLocation1 = null;
        packageName = null;
        newSymbolTable = new SymbolTable(null);
        newSymbolTable.beginBuild(0, 0);
        if (generateSyntaxErrorInfo || generateSemanticErrorInfo) {
            if (newErrorTable == null) {
                newErrorTable = new ErrorTable();
            } else {
                newErrorTable.clear();
            }
        }
        newImports = new LinkedList();
        newImports.add("java.lang.*");
    }


    private synchronized void postParse() {
        newSymbolTable.endBuild(Integer.MAX_VALUE, Integer.MAX_VALUE);
        while (newSymbolTable.getSuperTable() != null) {
            newSymbolTable = newSymbolTable.getSuperTable();
        }
        symbolTable = newSymbolTable;
        newSymbolTable = null;
        if (generateSyntaxErrorInfo || generateSemanticErrorInfo) {
            errorTable.replace(getPath(), newErrorTable);
        }
        imports = newImports;
        newImports = null;
        symbolPath.clear();
    }


    public ClassInfo getEnclosingClass(int line, int col) {
        List symbolPath = symbolTable.getSymbolPath(line, col);
        for (Iterator i = symbolPath.iterator(); i.hasNext(); ) {
            MemberInfo mi = (MemberInfo) i.next();
            if (mi instanceof ClassInfo) { return (ClassInfo) mi; }
        }
        return null;
    }


    public String parseExpression(String expr, SourceLocation sl,
                                  JavaParser baseParser) {
        Reader r = new StringReader(expr);
        init(r, sl.path);
        ReInit(r);
        classTable = baseParser.classTable;
        symbolPath = baseParser.symbolTable.getSymbolPath(sl.line, sl.col);
        symbolTable = newSymbolTable = baseParser.symbolTable;
        imports = newImports = baseParser.imports;

        // pretend to start at the specified line and column
        offsetLine = sl.line - 1;
        offsetCol = sl.col;

        String type = "";
        staticPrimary = null;

        try {
            type = Expression();
        } catch(ParseException pex) {
        } catch(TokenMgrError tme) {
        }

        if(     (!type.equals(""))
            &&  (staticPrimary != null)
            &&  (staticPrimary == getToken(0))
        ) {
            type = "*" + type;
        }

        return type;
    }


    /* *
    * Returns the symbol table, or <code>null</code> if this information
    * is not available.
    *
    public SymbolTable getSymbolTable() {
        return symbolTable;
    }
    */

    /**
    * Gets the set of packages which are imported by the code being parsed,
    * or <code>null</code> if the code has not yet been parsed.
    * If the code is part of a package, that package will be included in
    * the imports list.
    **/
    public synchronized String[] getImports() {
        return imports == null? null :
            (String[]) imports.toArray(new String[imports.size()]);
    }


    /**
    * Check if there is a comment before the current token, and if so,
    * collect the whole javadoc comment token sequence.
    **/
    private void checkComment() {
        Token t = getToken(0);
        if (t.specialToken != null) {
            comment = getJavaDocComment(t);
        }
    }


    /**
    * Given a Token which is the start of some symbol declaration, look
    * for a javadoc comment proceeding the token.
    */
    private static String getJavaDocComment(Token t) {
        Token st = t.specialToken;
        if(st == null) { return null; }
        String comment = "";
        while (st.specialToken != null) { st = st.specialToken; }
        while (st != null) {
            comment += st.image;
            st = st.next;
        }
        return comment;
    }


    /**
    * Gets the full class name of the enclosing class, if any.
    **/
    private final String getDeclaringClass() {
        for (Iterator i = symbolPath.iterator(); i.hasNext(); ) {
            MemberInfo s = (MemberInfo) i.next();
            if (s instanceof ClassInfo) {
                return ((ClassInfo) s).getFullName();
            }
        }
        return null;
    }


    private final ClassInfo getDeclaringClassInfo() {
        for (Iterator i = symbolPath.iterator(); i.hasNext(); ) {
            MemberInfo s = (MemberInfo) i.next();
            if (s instanceof ClassInfo) {
                return (ClassInfo) s;
            }
        }
        return null;
    }


    /**
    * Tries to turn a simple class name into a fully-qualified name.
    * Returns the original name if it fails.
    **/
    private final String resolveClassName(String name) {
        int dollar = name.lastIndexOf('.');
        if(dollar >= 0) {
            String potentialName = name.substring(0, dollar) + '$' +
                             name.substring(dollar+1);
            potentialName = resolveClassName(potentialName);
            if (classTable.get(potentialName) != null) {
                return potentialName;
            }
        }

        String arrayPart = "";
        String nonArrayPart = name;
        if(name.endsWith("[]")) {
            arrayPart = name.substring(name.lastIndexOf(']')+1);
            nonArrayPart = name.substring(0, name.indexOf('['));
        }

        // Check all the imports.
        for(Iterator i = newImports.iterator(); i.hasNext(); ) {
            String is = (String) i.next();
            if(is.endsWith(".*")) {
                String potentialName = is.substring(0, is.length() - 1) + name;
                ClassInfo ci = classTable.get(potentialName);
                if(ci != null) {
                    if (classTable.isAccessAllowed(getDeclaringClassInfo(),
                            ci, false, true))
                    {
                        return ci.getFullName();
                    }
                }
            } else if (is.endsWith("." + nonArrayPart)) {
                return is + arrayPart;
            }
            else if (is.endsWith("$" + nonArrayPart)) {
                return is + arrayPart;
            }
        }

        // Check enclosing classes or nearby inner classes.
        for(Iterator i = symbolPath.iterator(); i.hasNext(); ) {
            MemberInfo s = (MemberInfo) i.next();
            if(s instanceof ClassInfo) {
                String cName = ((ClassInfo)s).getFullName();
                if(cName.endsWith("." + nonArrayPart)) return cName + arrayPart;
                if(cName.endsWith("$" + nonArrayPart)) return cName + arrayPart;
                String potentialName = cName + '$' + name;
                ClassInfo ci = classTable.get(potentialName);
                if(ci != null) return ci.getFullName();
            }
        }

        return name + arrayPart;
    }


    private MemberInfo findPrimaryType(Token[] nameTokens, String[] args) {
        boolean staticReference = true;
        String name = nameTokens[0].image;
        int i = 0;
        MemberInfo mi = findSymbol(nameTokens[0],
            (nameTokens.length == 1? args : null)
        );
        if (mi != null) {
            staticReference = false;
            i++;
        } else {
            for (i = nameTokens.length-1; mi == null && i >= 0; i--) {
                name = nameTokens[0].image;
                for(int j = 1; j <= i; j++) {
                    name += "." + nameTokens[j].image;
                }
                mi = classTable.get(resolveClassName(name));
                if(mi != null) { i += 2; }
            }
        }
        if (i < 0) { i = nameTokens.length; }

        for(; i < nameTokens.length; i++) {
            name += "." + nameTokens[i].image;
            if(mi != null) {
                String type = resolveClassName(mi.getType());
                mi = findPrimaryType(type, type, null, staticReference,
                nameTokens[i], (i == nameTokens.length-1? args : null));
            }
            staticReference = false;
        }

        if (args != null && mi instanceof ClassInfo) { mi = null; }

        return mi;
    }


    private MemberInfo findSymbol(Token token, String[] args) {
        String name = token.image;
        int line = token.beginLine + offsetLine;
        int col = token.beginColumn;
        if (token.beginLine == 1) { col += offsetCol; }

        MemberInfo[] symbols;
        if(symbolTable != null) {
            symbols = symbolTable.getSymbols(name, line, col);
        } else {
            symbols = newSymbolTable.getSymbols(name, line, col);
        }
        MemberInfo sym = null;
        for(int i = 0; i < symbols.length; i++) {
            if(args != null) {
                if(symbols[i] instanceof MethodInfo) {
                    if(sym == null) {
                        sym = symbols[i];
                    } else if (matchMethodParams(((MethodInfo)symbols[i]), args)) {
                        sym = symbols[i];
                    }
                }
            } else {
                if (symbols[i] instanceof FieldInfo) { sym = symbols[i]; }
            }
        }

        if (generateSemanticErrorInfo && sym != null && sym instanceof MethodInfo &&
            !matchMethodParams(((MethodInfo)sym), args)
        ) {
            error(token, "Invalid parameters for method " + name + "().");
        }
        return sym;
    }


    private MemberInfo findPrimaryType(String prefixType, String originalType,
        MemberInfo found, boolean staticReference, Token memberToken,
        String[] args)
    {
        if (prefixType.equals("")) {
            return null;
        } else if(isPrimitiveType(prefixType)) {
            if(generateSemanticErrorInfo) {
                if(args == null) {
                    error(memberToken,
                        "Attempt to reference a field of primitive type " +
                        prefixType
                    );
                } else {
                    error(memberToken,
                        "Attempt to invoke a method on primitive type " +
                        prefixType);
                }
            }
            return null;
        }

        ClassInfo prefixClass = classTable.get(prefixType);
        if(prefixClass != null) {
            String memberName = memberToken.image;
            if(args == null) {
                for(Iterator i=prefixClass.getFields().iterator();i.hasNext();){
                    FieldInfo fi = (FieldInfo) i.next();
                    if(fi.getName().equals(memberName)) {
                        if(generateSemanticErrorInfo && staticReference &&
                            (fi.getModifiers() & Modifier.STATIC) == 0)
                        {
                            error(memberToken, "Non-static field "
                                + memberName
                                + " cannot be referenced from a static context."
                            );
                        }
                        return fi;
                    }
                }
                String superclass = prefixClass.getSuperclass();
                if(superclass != null) {
                    return findPrimaryType(superclass, originalType, found,
                                 staticReference, memberToken, args);
                }
                if(generateSemanticErrorInfo) {
          /*
          for(Iterator i = prefixClass.getMethods().iterator(); i.hasNext(); ) {
            MethodInfo mi = (MethodInfo) i.next();
            if(mi.getName().equals(memberName)) {
              error(memberToken, memberToken, "Attempt to reference method " +
                    memberName + " in class " + prefixType +
                    " as an instance variable.");
              return "";
            }
          }
          */
                    ClassInfo originalClass = classTable.get(originalType);
                    String debug ="";
                    for (Iterator i=originalClass.getFields().iterator();i.hasNext();){
                        FieldInfo fi = (FieldInfo) i.next();
                        debug += fi.getName() + ",";
                    }
                    error(memberToken, "No variable " + memberName +
                        " declared in " + originalType + "." + debug);
                }
                return null;
            } else { // Checking for methods
                for(Iterator i=prefixClass.getMethods().iterator();i.hasNext();)
                {
                    MethodInfo mi = (MethodInfo) i.next();
                    if(mi.getName().equals(memberName)) {
                        if(generateSemanticErrorInfo && staticReference &&
                        found == null && (mi.getModifiers() & Modifier.STATIC) == 0)
                        {
                            error(memberToken, "Non-static method " + memberName + "()" +
                            " cannot be referenced from a static context.");
                        }
                        if(matchMethodParams(mi, args)) {
                            return mi;
                        } else {
                            found = mi;
                        }
                    }
                }
                String superclass = prefixClass.getSuperclass();
                if(superclass != null) {
                    return findPrimaryType(superclass, originalType, found,
                                 staticReference, memberToken, args);
                }
                if(found != null) {
                    if(generateSemanticErrorInfo) {
                        error(memberToken, "Invalid parameters for method " +
                            memberName + "().");
                    }
                    return found;
                } else {
                    if(generateSemanticErrorInfo) {
                        error(memberToken, "Method " + memberName + "()" +
                            " not found in class " + originalType + ".");
                    }
                    return null;
                }
            }
        }
        return null;
    }


    private boolean matchMethodParams(MethodInfo mi, String[] args) {
        String[] paramTypes = mi.getParameterTypes();
        if(paramTypes.length != args.length) { return false; }
        for (int i = 0; i < args.length; i++) {
            if (!classTable.isAssignableTo(args[i], paramTypes[i], true)) {
                Log.log(Log.DEBUG, this, args[i] + " not assignable to " + paramTypes[i]);
                return false;
            }
        }
        return true;
    }


    private SourceLocation getSourceLocation() {
        return getSourceLocation(getToken(0));
    }


    private SourceLocation getSourceLocation(Token token) {
        return new SourceLocation(getPath(), token.beginLine, token.beginColumn,
            token.endLine, token.endColumn);
    }


    private SourceLocation getSourceLocation(Token startToken, Token endToken) {
        return new SourceLocation(getPath(),
            startToken.beginLine, startToken.beginColumn,
            endToken.endLine, endToken.endColumn
        );
    }


    private void error(Token tStart, Token tEnd, String msg) {
        newErrorTable.put(new ErrorInfo(getSourceLocation(tStart, tEnd), msg));
    }


    private void error(Token t, String msg) {
        newErrorTable.put(new ErrorInfo(getSourceLocation(t), msg));
    }


    private void error(SourceLocation location, String msg) {
        newErrorTable.put(new ErrorInfo(location, msg));
    }


    private void enterBlock(Token firstToken) {
        SymbolTable st = new SymbolTable(newSymbolTable);
        st.beginBuild(firstToken.beginLine, firstToken.beginColumn);
        newSymbolTable = st;
    }


    private void leaveBlock(Token lastToken) {
        newSymbolTable.endBuild(lastToken.endLine, lastToken.endColumn);
        newSymbolTable = newSymbolTable.getSuperTable();
    }


    private String booleanBinaryExpressionResult(
        String type1, Token e1Start, Token e1End,
        String type2, Token e2Start, Token e2End
    ) {
        if(type2 == null) {
            return type1;
        }
        else {
            if(generateSemanticErrorInfo) {
                if(!type1.equals("") && !type1.equals("boolean")) {
                    error(e1Start, e1End, "Expression must be of type boolean.");
                }
                if(!type2.equals("") && !type2.equals("boolean")) {
                    error(e1Start, e1End, "Expression must be of type boolean.");
                }
            }
            return "boolean";
        }
    }


    private static boolean isIntegralType(String type) {
        return type.equals("byte")
            || type.equals("char")
            || type.equals("short")
            || type.equals("int")
            || type.equals("long");
    }


    private static boolean isFloatType(String type) {
        return type.equals("float")
            || type.equals("double");
    }


    private static boolean isNumericType(String type) {
        return isIntegralType(type) || isFloatType(type);
    }


    private static boolean isPrimitiveType(String type) {
        return isNumericType(type) || type.equals("boolean") || type.equals("void");
    }


    private static String binaryPromoteLeft(String type1, String type2) {
        if (type1.equals("double") || (type2.equals("double") && isNumericType(type1))
        ) {
            return "double";
        } else if (type1.equals("float") || (type2.equals("float") && isIntegralType(type1))
        ) {
            return "float";
        } else if (type1.equals("long") || (type2.equals("long") && isIntegralType(type1))
        ) {
            return "long";
        } else if (isIntegralType(type1)) {
            return "int";
        } else {
            return type1;
        }
    }


    private static String binaryPromoteRight(String type1, String type2) {
        return binaryPromoteLeft(type2, type1);
    }


    private static String unaryPromote(String type) {
        if (type.equals("byte") || type.equals("char") || type.equals("short")) {
            return "int";
        } else {
            return type;
        }
    }


    private String logicalBinaryExpressionResult(String type1, Token e1Start,
                                                 Token e1End, String type2,
                                                 Token e2Start, Token e2End) {
        String errMsg =
            "Logical expression half must be of integral type or boolean.";
        if (type2 == null) {
            return type1;
        }
        else {
            if (isIntegralType(type1)) {
                if (type1.equals("long") || type2.equals("long")) {
                    type1 = "long";
                } else {
                    type1 = "int";
                }
            }
            if(isIntegralType(type2)) {
                if (type2.equals("long") || type1.equals("long")) {
                    type2 = "long";
                } else {
                    type2 = "int";
                }
            }

            if (type1.equals("")) {
                if (!type2.equals("") &&
                    !isIntegralType(type2) && !type2.equals("boolean")
                ) {
                    if (generateSemanticErrorInfo) {
                        error(e2Start, e2End, errMsg);
                    }
                    return "";
                } else {
                    return type2;
                }
            } else if (type2.equals("")) {
                if (!type1.equals("") &&
                    !isIntegralType(type1) && !type1.equals("boolean")
                ) {
                    if (generateSemanticErrorInfo) {
                        error(e1Start, e1End, errMsg);
                    }
                    return "";
                } else {
                    return type1;
                }
            } else {
                String ret = "";
                if (!isIntegralType(type1) && !type1.equals("boolean")) {
                    if (generateSemanticErrorInfo) {
                        error(e1Start, e1End, errMsg);
                    }
                    if (isIntegralType(type2) || type2.equals("boolean")) {
                        ret = type2;
                    }
                }
                if (!isIntegralType(type2) && !type2.equals("boolean")) {
                    if (generateSemanticErrorInfo) {
                        error(e2Start, e2End, errMsg);
                    }
                    if (isIntegralType(type1) || type1.equals("boolean")) {
                        ret = type1;
                    }
                }

                if (type1.equals(type2)) {
                    ret = type1;
                } else if (generateSemanticErrorInfo) {
                    error(e2Start, e2End, "Logical expression half types don't match.");
                }

                return ret;
            }
        }
    }


    private String equalityBinaryExpressionResult(String type1, Token e1Start,
                                                  Token e1End, String type2,
                                                  Token e2Start, Token e2End) {
        if (type2 == null) {
            return type1;
        } else {
            type1 = binaryPromoteLeft(type1, type2);
            type2 = binaryPromoteRight(type1, type2);
            if (generateSemanticErrorInfo && !type1.equals("") && !type2.equals("") &&
                !((isNumericType(type1) && isNumericType(type2)) ||
                (type1.equals("boolean") && type2.equals("boolean")) ||
                (!isPrimitiveType(type1) && !isPrimitiveType(type2)))
            ) {
                error(e2Start, e2End, "Equality expression halves must be " +
                    "both boolean, both numeric, or both reference types.");
            }
            return "boolean";
        }
    }


    private String comparisonBinaryExpressionResult(String type1, Token e1Start,
                                                    Token e1End, String type2,
                                                    Token e2Start, Token e2End) {
        if (type2 == null) {
            return type1;
        } else {
            type1 = binaryPromoteLeft(type1, type2);
            type2 = binaryPromoteRight(type1, type2);
            if (generateSemanticErrorInfo &&
                !type1.equals("") && !isNumericType(type1)
            ) {
                error(e1Start, e1End,
                    "Comparison expression half must be of numeric type1 [" + type1 + "].");
            }
            if (generateSemanticErrorInfo &&
                !type2.equals("") && !isNumericType(type2)
            ) {
                error(e2Start, e2End,
                    "Comparison expression half must be of numeric type2 [" + type2 + "].");
            }
            return "boolean";
        }
    }


    private String shiftBinaryExpressionResult(String type1, Token e1Start,
                                               Token e1End, String type2,
                                               Token e2Start, Token e2End) {
        if (type2 == null) {
            return type1;
        } else {
            type1 = unaryPromote(type1);
            type2 = unaryPromote(type2);
            if (generateSemanticErrorInfo &&
                !type1.equals("") && !isIntegralType(type1)
            ) {
                error(e1Start, e1End,
                    "Shift expression half must be of integral type.");
            }
            if (generateSemanticErrorInfo &&
                !type2.equals("") && !isIntegralType(type2)
            ) {
                error(e2Start, e2End,
                    "Shift expression half must be of integral type.");
            }
            if (!isIntegralType(type1)) {
                return "int";
            } else {
                return type1;
            }
        }
    }


    private String mathBinaryExpressionResult(String type1, Token e1Start,
                                              Token e1End, String type2,
                                              Token e2Start, Token e2End) {
        if (type2 == null) {
            return type1;
        } else {
            type1 = binaryPromoteLeft(type1, type2);
            type2 = binaryPromoteRight(type1, type2);
            if (generateSemanticErrorInfo &&
                !type1.equals("") && !isNumericType(type1)
            ) {
                error(e1Start, e1End,
                    "Math expression half must be of numeric type.");
            }
            if (generateSemanticErrorInfo &&
                !type2.equals("") && !isNumericType(type2)
            ) {
                error(e2Start, e2End,
                    "Math expression half must be of numeric type.");
            }
            if (isNumericType(type1)) {
                return type1;
            } else if (isNumericType(type2)) {
                return type2;
            } else {
                return "";
            }
        }
    }


    private String numericUnaryExpressionResult(String type, Token eStart,
                                                Token eEnd) {
        if (!type.equals("") && !isNumericType(type)) {
            if (generateSemanticErrorInfo) {
                error(eStart, eEnd, "Unary expression must be of numeric type.");
            }
            type = "";
        }
        return type;
    }


    private String integralUnaryExpressionResult(String type, Token eStart,
                                                 Token eEnd) {
        if (!type.equals("")) {
            if (!isIntegralType(type)) {
                if (generateSemanticErrorInfo) {
                    error(eStart, eEnd, "Unary expression must be of integral type.");
                }
                type = "";
            } else {
                type = unaryPromote(type);
            }
        }
        return type;
    }


    private String booleanUnaryExpressionResult(String type, Token eStart,
                                                Token eEnd) {
        if (!type.equals("")) {
            if (!type.equals("boolean")) {
                if (generateSemanticErrorInfo) {
                    error(eStart, eEnd, "Unary expression must be of type boolean.");
                }
                type = "";
            }
        }
        return type;
    }


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * All code below is from Java1.1noLA from the JavaCC examples.
 * It is Copyright (C) 1996, 1997 Sun Microsystems Inc., and is
 * re-used according to the JavaCC license.
 * Author: Sriram Sankar
 * Date: 9/11/97
 *
 * Actions have been added to this code to facilitate recording of
 * declared fields, methods, and constructors.
 */


  // Semantic lookahead rules follow.

  // implements:  LOOKAHEAD( ( "abstract" | "final" | "public" )* "class" )
  final boolean isClassDeclaration() {
    int curTok = 1;
    Token tok;
    while (true) {
      tok = getToken(curTok);
      switch (tok.kind) {
      case ABSTRACT:
      case FINAL:
      case PUBLIC:
        curTok++;
        break;
      case CLASS:
        return true;
      default:
        return false;
      }
    }
  }

  // implements:  LOOKAHEAD(2)  for Initializer()
  final boolean isInitializer() {
    Token tok = getToken(1);
    if (tok.kind == LBRACE) return true;
    if (tok.kind == STATIC) {
      tok = getToken(2);
      if (tok.kind == LBRACE) return true;
    }
    return false;
  }

  // implements:  LOOKAHEAD( ( "static" | "abstract" | "final" | "public" | "protected" | "private" )* "class" )
  final boolean isNestedClassDeclaration() {
    int curTok = 1;
    Token tok;
    while (true) {
      tok = getToken(curTok);
      switch (tok.kind) {
      case STATIC:
      case ABSTRACT:
      case FINAL:
      case PUBLIC:
      case PROTECTED:
      case PRIVATE:
        curTok++;
        break;
      case CLASS:
        return true;
      default:
        return false;
      }
    }
  }

  // implements:  LOOKAHEAD( ( "static" | "abstract" | "final" | "public" | "protected" | "private" )* "interface" )
  final boolean isNestedInterfaceDeclaration() {
    int curTok = 1;
    Token tok;
    while (true) {
      tok = getToken(curTok);
      switch (tok.kind) {
      case STATIC:
      case ABSTRACT:
      case FINAL:
      case PUBLIC:
      case PROTECTED:
      case PRIVATE:
        curTok++;
        break;
      case INTERFACE:
        return true;
      default:
        return false;
      }
    }
  }

  // implements:  LOOKAHEAD( [ "public" | "protected" | "private" ] Name() "(" )
  final boolean isConstructorDeclaration() {
    int curTok = 1;
    Token tok;
    tok = getToken(1);
    switch (tok.kind) {
    case PUBLIC:
    case PROTECTED:
    case PRIVATE:
      curTok = 2;
    }
    tok = getToken(curTok++);
    if (tok.kind != IDENTIFIER) return false;
    while (true) {
      tok = getToken(curTok++);
      if (tok.kind == LPAREN) return true;
      if (tok.kind != DOT) return false;
      tok = getToken(curTok++);
      if (tok.kind != IDENTIFIER) return false;
    }
  }

  // Returns true if the next set of tokens cannot be a field declaration.
  // Returns false if the next set of tokens cannot be a method declaration.
  // Note how this method is used in the grammar.  We don't have to do a
  // comprehensive check like we have done in the lookahead methods above.
  // This show (therefore) another way you can implement lookahead methods.
  // The way we do it is to see if we can find a "(" before either a "=" or
  // a ";", in which case we return true.
  final boolean isMethodDeclaration() {
    int curTok = 1;
    Token tok;
    while (true) {
      tok = getToken(curTok++);
      switch (tok.kind) {
      case LPAREN:
        return true;
      case ASSIGN:
      case SEMICOLON:
      case EOF:
        return false;
      }
    }
  }

  // Checks that there is a "," and then there is no "}" following that.
  final boolean moreVariableInitializers() {
    return getToken(1).kind == COMMA && getToken(2).kind != RBRACE;
  }

  // Checks that this is a constructor invocation as opposed to a block
  // statement.
  final boolean isConstructorInvocation() {
    int curTok = 1;
    Token tok = getToken(1);
    switch (tok.kind) {
    case THIS:
    case SUPER:
      // We are assuming here that if the statement starts with "this"
      // or "super", and the next token is "(", then it has to be a
      // constructor invocation.
      return getToken(2).kind == LPAREN;
    case STRING_LITERAL:
    case LPAREN:
    case NEW:
    case IDENTIFIER:
      // Now move across tokens until the end of the statement - the
      // first semicolon not nested within any kind of parentheses.
      // If a "super(" is matched also not nested and before this
      // semicolon, we return true.  Otherwise return false.
      int nestingLevel = 0;
      while (true) {
        tok = getToken(curTok++);
        switch (tok.kind) {
        case SEMICOLON:
          if (nestingLevel == 0) {
            return false;
          }
          break;
        case SUPER:
          if (nestingLevel == 0) {
            return getToken(curTok).kind == LPAREN;
          }
          break;
        case LPAREN:
        case LBRACE:
        case LBRACKET:
          nestingLevel++;
          break;
        case RPAREN:
        case RBRACE:
        case RBRACKET:
          nestingLevel--;
          break;
        case EOF:
          return false;
        }
      }
    default:
      return false;
    }
  }

  // Returns true if this is a primitive type (or an array of primitive
  // type) cast.
  final boolean isPrimitiveTypeCast() {
    if (getToken(1).kind != LPAREN) return false;
    Token tok = getToken(2);
    switch (tok.kind) {
    case BOOLEAN:
    case CHAR:
    case BYTE:
    case SHORT:
    case INT:
    case LONG:
    case FLOAT:
    case DOUBLE:
      return true;
    }
    return false;
  }

  // Returns true if this is a type cast.
  final boolean isTypeCast() {
    if (isPrimitiveTypeCast()) return true;
    if (getToken(1).kind != LPAREN) return false;
    int curTok = 2;
    Token tok = getToken(curTok++);
    if (tok.kind != IDENTIFIER) return false;
    while (true) {
      tok = getToken(curTok++);
      if (tok.kind != DOT) break;
      tok = getToken(curTok++);
      if (tok.kind != IDENTIFIER) return false;
    }
    if (tok.kind == RPAREN) {
      tok = getToken(curTok);
      switch (tok.kind) {
      case TILDE:
      case BANG:
      case LPAREN:
      case IDENTIFIER:
      case THIS:
      case SUPER:
      case NEW:
      case INTEGER_LITERAL:
      case FLOATING_POINT_LITERAL:
      case CHARACTER_LITERAL:
      case STRING_LITERAL:
      case TRUE:
      case FALSE:
      case NULL:
        return true;
      }
      return false;
    } else {
      if (tok.kind != LBRACKET) return false;
      tok = getToken(curTok);
      if (tok.kind != RBRACKET) return false;
      return true;
    }
  }

  // Distinguishes between ClassSelector and Name.
  final boolean isClassSelector() {
    int curTok = 1;
    Token tok = getToken(curTok++);
    if (tok.kind != IDENTIFIER) return true;
    while (true) {
      tok = getToken(curTok++);
      while (tok.kind == LBRACKET) {
        // A simple loop to accept "[]"s.  We are a little sloppy
        // in that we don't require it to be at the end, but then
        // this is only a lookahead check.
        tok = getToken(curTok++);
        if (tok.kind != RBRACKET) return false;
        tok = getToken(curTok++);
      }
      if (tok.kind != DOT) return false;
      tok = getToken(curTok++);
      if (tok.kind == CLASS) return true;
      if (tok.kind != IDENTIFIER) return false;
    }
  }

  // implements:  LOOKAHEAD([ "final" ] Type() <IDENTIFIER>)
  final boolean isLocalVariableDeclaration() {
    int curTok = 1;
    Token tok = getToken(curTok++);
    if (tok.kind == FINAL) tok = getToken(curTok++);
    switch (tok.kind) {
    case BOOLEAN:
    case CHAR:
    case BYTE:
    case SHORT:
    case INT:
    case LONG:
    case FLOAT:
    case DOUBLE:
      tok = getToken(curTok++);
      break;
    case IDENTIFIER:
      while (true) {
        tok = getToken(curTok++);
        if (tok.kind != DOT) break;
        tok = getToken(curTok++);
        if (tok.kind != IDENTIFIER) return false;
      }
      break;
    default:
      return false;
    }
    while (tok.kind == LBRACKET) {
      tok = getToken(curTok++);
      if (tok.kind != RBRACKET) return false;
      tok = getToken(curTok++);
    }
    return tok.kind == IDENTIFIER;
  }

  final boolean isPrimarySuffix() {
    Token tok = getToken(1);
    if(tok.kind == LBRACKET) return true;
    else if(tok.kind == DOT && getToken(2).kind == IDENTIFIER) return true;
    else return false;
  }

  final boolean isThis() {
    Token tok = getToken(1);
    for(int i = 1; ; i+=2) {
      tok = getToken(i);
      if(tok.kind == THIS) return true;
      else if(tok.kind != IDENTIFIER) break;
      if(getToken(i+1).kind != DOT) break;
    }
    return false;
  }

/*****************************************
 * THE JAVA LANGUAGE GRAMMAR STARTS HERE *
 *****************************************/

/*
 * Program structuring syntax follows.
 */
  final public void CompilationUnit() throws ParseException {
    switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
    case PACKAGE:
      PackageDeclaration();
      break;
    default:
      jj_la1[0] = jj_gen;
      ;
    }
    label_1:
    while (true) {
      switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
      case IMPORT:
        ;
        break;
      default:
        jj_la1[1] = jj_gen;
        break label_1;
      }
      ImportDeclaration();
    }
    label_2:
    while (true) {
      switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
      case ABSTRACT:
      case CLASS:
      case FINAL:
      case INTERFACE:
      case PUBLIC:
      case SEMICOLON:
        ;
        break;
      default:
        jj_la1[2] = jj_gen;
        break label_2;
      }
      TypeDeclaration();
    }
    jj_consume_token(0);
  }

  final public void PackageDeclaration() throws ParseException {
    jj_consume_token(PACKAGE);
    packageName = Name();
    jj_consume_token(SEMICOLON);
    newImports.add(packageName + ".*");
  }

  final public void ImportDeclaration() throws ParseException {
  String name; boolean star = false;
    jj_consume_token(IMPORT);
    name = Name();
    switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
    case DOT:
      jj_consume_token(DOT);
      jj_consume_token(STAR);
                                   star = true;
      break;
    default:
      jj_la1[3] = jj_gen;
      ;
    }
    jj_consume_token(SEMICOLON);
    if(star) newImports.add(name + ".*");
    else {
      int dollar = name.lastIndexOf('.');
      if(dollar >= 0) {
        String icName = name.substring(0, dollar) + '$' +
                        name.substring(dollar+1);
        if(classTable.get(icName) != null) name = icName;
      }
      newImports.add(0, name);
    }
  }

  final public void TypeDeclaration() throws ParseException {
    if (isClassDeclaration()) {
      ClassDeclaration();
    } else {
      switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
      case ABSTRACT:
      case INTERFACE:
      case PUBLIC:
        InterfaceDeclaration();
        break;
      case SEMICOLON:
        jj_consume_token(SEMICOLON);
        break;
      default:
        jj_la1[4] = jj_gen;
        jj_consume_token(-1);
        throw new ParseException();
      }
    }
  }

/*
 * Declaration syntax follows.
 */
  final public void ClassDeclaration() throws ParseException {
    label_3:
    while (true) {
      switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
      case ABSTRACT:
      case FINAL:
      case PUBLIC:
        ;
        break;
      default:
        jj_la1[5] = jj_gen;
        break label_3;
      }
      switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
      case ABSTRACT:
        jj_consume_token(ABSTRACT);
                   modifiers = modifiers | Modifier.ABSTRACT;
        break;
      case FINAL:
        jj_consume_token(FINAL);
                   modifiers = modifiers | Modifier.FINAL;
        break;
      case PUBLIC:
        jj_consume_token(PUBLIC);
                   modifiers = modifiers | Modifier.PUBLIC;
        break;
      default:
        jj_la1[6] = jj_gen;
        jj_consume_token(-1);
        throw new ParseException();
      }
      checkComment();
    }
    UnmodifiedClassDeclaration();
  }

  final public void UnmodifiedClassDeclaration() throws ParseException {
  Token t; String name; String superclass = null;
  String[] interfaces = null; ClassInfo ci;
    jj_consume_token(CLASS);
    checkComment();
    t = jj_consume_token(IDENTIFIER);
    switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
    case EXTENDS:
      jj_consume_token(EXTENDS);
      superclass = Name();
      break;
    default:
      jj_la1[7] = jj_gen;
      ;
    }
    switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
    case IMPLEMENTS:
      jj_consume_token(IMPLEMENTS);
      interfaces = NameList();
      break;
    default:
      jj_la1[8] = jj_gen;
      ;
    }
    sourceLocation = getSourceLocation(t);
    name = t.image;
    t = getToken(1);

    if(superclass == null && !(name.equals("Object") &&
       packageName != null && packageName.equals("java.lang")))
      superclass = "java.lang.Object";

    if(superclass != null) superclass = resolveClassName(superclass);

    if(interfaces == null) interfaces = new String[0];
    for(int i = 0; i < interfaces.length; i++)
      interfaces[i] = resolveClassName(interfaces[i]);

    ci = new ClassInfo(getDeclaringClass(), modifiers, false, name,
                       superclass, interfaces, packageName, comment);
    ci.setSourceLocation(sourceLocation);
    if(oneLineComments) ci.setComment(ci.getOneLineComment());

    newSymbolTable.addSymbol(ci);
    enterBlock(t);
    newSymbolTable.setSymbol(ci);

    if(symbolPath.size() > 0 && symbolPath.get(0) instanceof ClassInfo)
      ((ClassInfo)symbolPath.get(0)).add(ci);
    symbolPath.add(0, ci);

    comment = null;
    modifiers = 0;
    ClassBody();
    leaveBlock(getToken(0));

    symbolPath.remove(0);

    if(generateClassInfo) {
      if(generatePrivateClassInfo ||
         (ci.getModifiers() & Modifier.PUBLIC) != 0 ||
         (ci.getModifiers() & Modifier.PROTECTED) != 0) {
        classTable.put(ci);
      }
    }
  }

  final public void ClassBody() throws ParseException {
    jj_consume_token(LBRACE);
    label_4:
    while (true) {
      switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
      case ABSTRACT:
      case BOOLEAN:
      case BYTE:
      case CHAR:
      case CLASS:
      case DOUBLE:
      case FINAL:
      case FLOAT:
      case INT:
      case INTERFACE:
      case LONG:
      case NATIVE:
      case PRIVATE:
      case PROTECTED:
      case PUBLIC:
      case SHORT:
      case STATIC:
      case STRICTFP:
      case SYNCHRONIZED:
      case TRANSIENT:
      case VOID:
      case VOLATILE:
      case IDENTIFIER:
      case LBRACE:
        ;
        break;
      default:
        jj_la1[9] = jj_gen;
        break label_4;
      }
      ClassBodyDeclaration();
    }
    jj_consume_token(RBRACE);
  }

  final public void NestedClassDeclaration() throws ParseException {
    label_5:
    while (true) {
      switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
      case ABSTRACT:
      case FINAL:
      case PRIVATE:
      case PROTECTED:
      case PUBLIC:
      case STATIC:
        ;
        break;
      default:
        jj_la1[10] = jj_gen;
        break label_5;
      }
      switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
      case STATIC:
        jj_consume_token(STATIC);
                    modifiers = modifiers | Modifier.STATIC;
        break;
      case ABSTRACT:
        jj_consume_token(ABSTRACT);
                    modifiers = modifiers | Modifier.ABSTRACT;
        break;
      case FINAL:
        jj_consume_token(FINAL);
                    modifiers = modifiers | Modifier.FINAL;
        break;
      case PUBLIC:
        jj_consume_token(PUBLIC);
                    modifiers = modifiers | Modifier.PUBLIC;
        break;
      case PROTECTED:
        jj_consume_token(PROTECTED);
                    modifiers = modifiers | Modifier.PROTECTED;
        break;
      case PRIVATE:
        jj_consume_token(PRIVATE);
                    modifiers = modifiers | Modifier.PRIVATE;
        break;
      default:
        jj_la1[11] = jj_gen;
        jj_consume_token(-1);
        throw new ParseException();
      }
      checkComment();
    }
    UnmodifiedClassDeclaration();
  }

  final public void ClassBodyDeclaration() throws ParseException {
    if (isInitializer()) {
      Initializer();
    } else if (isNestedClassDeclaration()) {
      NestedClassDeclaration();
    } else if (isNestedInterfaceDeclaration()) {
      NestedInterfaceDeclaration();
    } else if (isConstructorDeclaration()) {
      ConstructorDeclaration();
    } else if (isMethodDeclaration()) {
      MethodDeclaration();
    } else {
      switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
      case BOOLEAN:
      case BYTE:
      case CHAR:
      case DOUBLE:
      case FINAL:
      case FLOAT:
      case INT:
      case LONG:
      case PRIVATE:
      case PROTECTED:
      case PUBLIC:
      case SHORT:
      case STATIC:
      case TRANSIENT:
      case VOLATILE:
      case IDENTIFIER:
        FieldDeclaration();
        break;
      default:
        jj_la1[12] = jj_gen;
        jj_consume_token(-1);
        throw new ParseException();
      }
    }
  }

  final public void InterfaceDeclaration() throws ParseException {
    label_6:
    while (true) {
      switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
      case ABSTRACT:
      case PUBLIC:
        ;
        break;
      default:
        jj_la1[13] = jj_gen;
        break label_6;
      }
      switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
      case ABSTRACT:
        jj_consume_token(ABSTRACT);
                    modifiers = modifiers | Modifier.ABSTRACT;
        break;
      case PUBLIC:
        jj_consume_token(PUBLIC);
                    modifiers = modifiers | Modifier.PUBLIC;
        break;
      default:
        jj_la1[14] = jj_gen;
        jj_consume_token(-1);
        throw new ParseException();
      }
      checkComment();
    }
    UnmodifiedInterfaceDeclaration();
  }

  final public void NestedInterfaceDeclaration() throws ParseException {
    label_7:
    while (true) {
      switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
      case ABSTRACT:
      case FINAL:
      case PRIVATE:
      case PROTECTED:
      case PUBLIC:
      case STATIC:
        ;
        break;
      default:
        jj_la1[15] = jj_gen;
        break label_7;
      }
      switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
      case STATIC:
        jj_consume_token(STATIC);
                    modifiers = modifiers | Modifier.STATIC;
        break;
      case ABSTRACT:
        jj_consume_token(ABSTRACT);
                    modifiers = modifiers | Modifier.ABSTRACT;
        break;
      case FINAL:
        jj_consume_token(FINAL);
                    modifiers = modifiers | Modifier.FINAL;
        break;
      case PUBLIC:
        jj_consume_token(PUBLIC);
                    modifiers = modifiers | Modifier.PUBLIC;
        break;
      case PROTECTED:
        jj_consume_token(PROTECTED);
                    modifiers = modifiers | Modifier.PROTECTED;
        break;
      case PRIVATE:
        jj_consume_token(PRIVATE);
                    modifiers = modifiers | Modifier.PRIVATE;
        break;
      default:
        jj_la1[16] = jj_gen;
        jj_consume_token(-1);
        throw new ParseException();
      }
      checkComment();
    }
    UnmodifiedInterfaceDeclaration();
  }

  final public void UnmodifiedInterfaceDeclaration() throws ParseException {
  Token t; String name; String[] interfaces = null; ClassInfo ci;
    jj_consume_token(INTERFACE);
    checkComment();
    t = jj_consume_token(IDENTIFIER);
    switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
    case EXTENDS:
      jj_consume_token(EXTENDS);
      interfaces = NameList();
      break;
    default:
      jj_la1[17] = jj_gen;
      ;
    }
    sourceLocation = getSourceLocation(t);
    name = t.image;
    t = getToken(1);

    if(interfaces == null) interfaces = new String[0];
    for(int i = 0; i < interfaces.length; i++)
      interfaces[i] = resolveClassName(interfaces[i]);

    ci = new ClassInfo(getDeclaringClass(), modifiers, true, name,
                       null, interfaces, packageName, comment);
    ci.setSourceLocation(sourceLocation);
    if(oneLineComments) ci.setComment(ci.getOneLineComment());

    comment = null;
    modifiers = 0;

    newSymbolTable.addSymbol(ci);
    enterBlock(t);
    newSymbolTable.setSymbol(ci);

    if(symbolPath.size() > 0 && symbolPath.get(0) instanceof ClassInfo)
      ((ClassInfo)symbolPath.get(0)).add(ci);
    symbolPath.add(0, ci);
    jj_consume_token(LBRACE);
    label_8:
    while (true) {
      switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
      case ABSTRACT:
      case BOOLEAN:
      case BYTE:
      case CHAR:
      case CLASS:
      case DOUBLE:
      case FINAL:
      case FLOAT:
      case INT:
      case INTERFACE:
      case LONG:
      case NATIVE:
      case PRIVATE:
      case PROTECTED:
      case PUBLIC:
      case SHORT:
      case STATIC:
      case STRICTFP:
      case SYNCHRONIZED:
      case TRANSIENT:
      case VOID:
      case VOLATILE:
      case IDENTIFIER:
        ;
        break;
      default:
        jj_la1[18] = jj_gen;
        break label_8;
      }
      InterfaceMemberDeclaration();
    }
    jj_consume_token(RBRACE);
    leaveBlock(getToken(0));

    symbolPath.remove(0);

    if(generateClassInfo) {
      if(generatePrivateClassInfo ||
         (ci.getModifiers() & Modifier.PUBLIC) != 0 ||
         (ci.getModifiers() & Modifier.PROTECTED) != 0) {
        classTable.put(ci);
      }
    }
  }

  final public void InterfaceMemberDeclaration() throws ParseException {
    if (isNestedClassDeclaration()) {
      NestedClassDeclaration();
    } else if (isNestedInterfaceDeclaration()) {
      NestedInterfaceDeclaration();
    } else if (isMethodDeclaration()) {
      MethodDeclaration();
    } else {
      switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
      case BOOLEAN:
      case BYTE:
      case CHAR:
      case DOUBLE:
      case FINAL:
      case FLOAT:
      case INT:
      case LONG:
      case PRIVATE:
      case PROTECTED:
      case PUBLIC:
      case SHORT:
      case STATIC:
      case TRANSIENT:
      case VOLATILE:
      case IDENTIFIER:
        FieldDeclaration();
        break;
      default:
        jj_la1[19] = jj_gen;
        jj_consume_token(-1);
        throw new ParseException();
      }
    }
  }

  final public void FieldDeclaration() throws ParseException {
  Token t; String type; String thisType; String name; FieldInfo fi;
    label_9:
    while (true) {
      switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
      case FINAL:
      case PRIVATE:
      case PROTECTED:
      case PUBLIC:
      case STATIC:
      case TRANSIENT:
      case VOLATILE:
        ;
        break;
      default:
        jj_la1[20] = jj_gen;
        break label_9;
      }
      switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
      case PUBLIC:
        jj_consume_token(PUBLIC);
                    modifiers = modifiers | Modifier.PUBLIC;
        break;
      case PROTECTED:
        jj_consume_token(PROTECTED);
                    modifiers = modifiers | Modifier.PROTECTED;
        break;
      case PRIVATE:
        jj_consume_token(PRIVATE);
                    modifiers = modifiers | Modifier.PRIVATE;
        break;
      case STATIC:
        jj_consume_token(STATIC);
                    modifiers = modifiers | Modifier.STATIC;
        break;
      case FINAL:
        jj_consume_token(FINAL);
                    modifiers = modifiers | Modifier.FINAL;
        break;
      case TRANSIENT:
        jj_consume_token(TRANSIENT);
                    modifiers = modifiers | Modifier.TRANSIENT;
        break;
      case VOLATILE:
        jj_consume_token(VOLATILE);
                    modifiers = modifiers | Modifier.VOLATILE;
        break;
      default:
        jj_la1[21] = jj_gen;
        jj_consume_token(-1);
        throw new ParseException();
      }
      checkComment();
    }
    type = Type();
    checkComment();
    name = VariableDeclarator();
    thisType = type;
    while(name.endsWith("[]")) {
      name = name.substring(0, name.length() - 2);
      thisType += "[]";
    }
    thisType = resolveClassName(thisType);
    fi = new FieldInfo(getDeclaringClass(), modifiers, thisType,
                       name, comment);
    fi.setSourceLocation(sourceLocation);
    if(oneLineComments) fi.setComment(fi.getOneLineComment());

    if(symbolPath.get(0) instanceof ClassInfo) {
      if(generatePrivateClassInfo ||
         (fi.getModifiers() & Modifier.PUBLIC) != 0 ||
         (fi.getModifiers() & Modifier.PROTECTED) != 0) {
        ((ClassInfo)symbolPath.get(0)).add(fi);
      }
    }
    newSymbolTable.addSymbol(fi);
    label_10:
    while (true) {
      switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
      case COMMA:
        ;
        break;
      default:
        jj_la1[22] = jj_gen;
        break label_10;
      }
      jj_consume_token(COMMA);
      name = VariableDeclarator();
      thisType = type;
      while(name.endsWith("[]")) {
        name = name.substring(0, name.length() - 2);
        thisType += "[]";
      }
      thisType = resolveClassName(thisType);
      fi = new FieldInfo(getDeclaringClass(), modifiers, thisType,
                         name, comment);
      fi.setSourceLocation(sourceLocation);
      if(oneLineComments) fi.setComment(fi.getOneLineComment());

      if(symbolPath.get(0) instanceof ClassInfo)
        ((ClassInfo)symbolPath.get(0)).add(fi);
      newSymbolTable.addSymbol(fi);
    }
    jj_consume_token(SEMICOLON);
    comment = null;
    modifiers = 0;
  }

  final public String VariableDeclarator() throws ParseException {
  String name;
    name = VariableDeclaratorId();
    switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
    case ASSIGN:
      jj_consume_token(ASSIGN);
      VariableInitializer();
      break;
    default:
      jj_la1[23] = jj_gen;
      ;
    }
    {if (true) return name;}
    throw new Error("Missing return statement in function");
  }

  final public String VariableDeclaratorId() throws ParseException {
  Token t; String name;
    t = jj_consume_token(IDENTIFIER);
                   name = t.image;
    label_11:
    while (true) {
      switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
      case LBRACKET:
        ;
        break;
      default:
        jj_la1[24] = jj_gen;
        break label_11;
      }
      jj_consume_token(LBRACKET);
      jj_consume_token(RBRACKET);
                                                 name += "[]";
    }
    sourceLocation = getSourceLocation(t);
    {if (true) return name;}
    throw new Error("Missing return statement in function");
  }

  final public void VariableInitializer() throws ParseException {
    switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
    case LBRACE:
      ArrayInitializer();
      break;
    case BOOLEAN:
    case BYTE:
    case CHAR:
    case DOUBLE:
    case FALSE:
    case FLOAT:
    case INT:
    case LONG:
    case NEW:
    case NULL:
    case SHORT:
    case SUPER:
    case THIS:
    case TRUE:
    case VOID:
    case INTEGER_LITERAL:
    case FLOATING_POINT_LITERAL:
    case CHARACTER_LITERAL:
    case STRING_LITERAL:
    case IDENTIFIER:
    case LPAREN:
    case BANG:
    case TILDE:
    case INCR:
    case DECR:
    case PLUS:
    case MINUS:
      Expression();
      break;
    default:
      jj_la1[25] = jj_gen;
      jj_consume_token(-1);
      throw new ParseException();
    }
  }

  final public void ArrayInitializer() throws ParseException {
    jj_consume_token(LBRACE);
    switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
    case BOOLEAN:
    case BYTE:
    case CHAR:
    case DOUBLE:
    case FALSE:
    case FLOAT:
    case INT:
    case LONG:
    case NEW:
    case NULL:
    case SHORT:
    case SUPER:
    case THIS:
    case TRUE:
    case VOID:
    case INTEGER_LITERAL:
    case FLOATING_POINT_LITERAL:
    case CHARACTER_LITERAL:
    case STRING_LITERAL:
    case IDENTIFIER:
    case LPAREN:
    case LBRACE:
    case BANG:
    case TILDE:
    case INCR:
    case DECR:
    case PLUS:
    case MINUS:
      VariableInitializer();
      label_12:
      while (true) {
        if (getToken(1).kind == COMMA && getToken(2).kind != RBRACE) {
          ;
        } else {
          break label_12;
        }
        jj_consume_token(COMMA);
        VariableInitializer();
      }
      break;
    default:
      jj_la1[26] = jj_gen;
      ;
    }
    switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
    case COMMA:
      jj_consume_token(COMMA);
      break;
    default:
      jj_la1[27] = jj_gen;
      ;
    }
    jj_consume_token(RBRACE);
  }

  final public void MethodDeclaration() throws ParseException {
  Token t=null; String type; String[] exceptionTypes=null; Object[] decl;
    t = getToken(1);
    if(t.specialToken != null)
      comment = getJavaDocComment(t);
    label_13:
    while (true) {
      switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
      case ABSTRACT:
      case FINAL:
      case NATIVE:
      case PRIVATE:
      case PROTECTED:
      case PUBLIC:
      case STATIC:
      case STRICTFP:
      case SYNCHRONIZED:
        ;
        break;
      default:
        jj_la1[28] = jj_gen;
        break label_13;
      }
      switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
      case PUBLIC:
        jj_consume_token(PUBLIC);
                       modifiers = modifiers | Modifier.PUBLIC;
        break;
      case PROTECTED:
        jj_consume_token(PROTECTED);
                       modifiers = modifiers | Modifier.PROTECTED;
        break;
      case PRIVATE:
        jj_consume_token(PRIVATE);
                       modifiers = modifiers | Modifier.PRIVATE;
        break;
      case STATIC:
        jj_consume_token(STATIC);
                       modifiers = modifiers | Modifier.STATIC;
        break;
      case ABSTRACT:
        jj_consume_token(ABSTRACT);
                       modifiers = modifiers | Modifier.ABSTRACT;
        break;
      case FINAL:
        jj_consume_token(FINAL);
                       modifiers = modifiers | Modifier.FINAL;
        break;
      case NATIVE:
        jj_consume_token(NATIVE);
                       modifiers = modifiers | Modifier.NATIVE;
        break;
      case SYNCHRONIZED:
        jj_consume_token(SYNCHRONIZED);
                       modifiers = modifiers | Modifier.SYNCHRONIZED;
        break;
      case STRICTFP:
        jj_consume_token(STRICTFP);

        break;
      default:
        jj_la1[29] = jj_gen;
        jj_consume_token(-1);
        throw new ParseException();
      }
    }
    type = ResultType();
    decl = MethodDeclarator();
    switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
    case THROWS:
      jj_consume_token(THROWS);
      exceptionTypes = NameList();
      break;
    default:
      jj_la1[30] = jj_gen;
      ;
    }
    t = getToken(1);

    type = resolveClassName(type);
    String[][] params = (String[][]) decl[1];
    for(int i = 0; i < params[0].length; i++)
      params[0][i] = resolveClassName(params[0][i]);
    if(exceptionTypes == null) exceptionTypes = new String[0];
    for(int i = 0; i < exceptionTypes.length; i++)
      exceptionTypes[i] = resolveClassName(exceptionTypes[i]);

    MethodInfo mi = new MethodInfo(getDeclaringClass(), modifiers,
        type, (String)decl[0], params[0], params[1], exceptionTypes, comment);
    mi.setSourceLocation(sourceLocation1);
    if(oneLineComments) mi.setComment(mi.getOneLineComment());
    comment = null;
    modifiers = 0;

    if(symbolPath.get(0) instanceof ClassInfo) {
      if(generatePrivateClassInfo ||
         (mi.getModifiers() & Modifier.PUBLIC) != 0 ||
         (mi.getModifiers() & Modifier.PROTECTED) != 0 ||
         ((ClassInfo)symbolPath.get(0)).isInterface()) {
        ((ClassInfo)symbolPath.get(0)).add(mi);
      }
    }
    symbolPath.add(0, mi);

    newSymbolTable.addSymbol(mi);
    enterBlock(t);
    newSymbolTable.setSymbol(mi);

    for(int i = 0; i < params[0].length; i++) {
      FieldInfo fi = new FieldInfo(getDeclaringClass(), 0,
                                   params[0][i], params[1][i], null);
      newSymbolTable.addSymbol(fi);
    }
    switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
    case LBRACE:
      Block();
      break;
    case SEMICOLON:
      jj_consume_token(SEMICOLON);
      break;
    default:
      jj_la1[31] = jj_gen;
      jj_consume_token(-1);
      throw new ParseException();
    }
    leaveBlock(getToken(0));
    symbolPath.remove(0);
  }

  final public Object[] MethodDeclarator() throws ParseException {
  Token t; String[][] params;
    t = jj_consume_token(IDENTIFIER);
    sourceLocation1 = getSourceLocation(t);
    params = FormalParameters();
    label_14:
    while (true) {
      switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
      case LBRACKET:
        ;
        break;
      default:
        jj_la1[32] = jj_gen;
        break label_14;
      }
      jj_consume_token(LBRACKET);
      jj_consume_token(RBRACKET);
    }
    {if (true) return new Object[] { t.image, params };}
    throw new Error("Missing return statement in function");
  }

  final public String[][] FormalParameters() throws ParseException {
  List typeL = new LinkedList(); List nameL = new LinkedList();
  String[] typeAndName;
    jj_consume_token(LPAREN);
    switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
    case BOOLEAN:
    case BYTE:
    case CHAR:
    case DOUBLE:
    case FINAL:
    case FLOAT:
    case INT:
    case LONG:
    case SHORT:
    case IDENTIFIER:
      typeAndName = FormalParameter();
        typeL.add(typeAndName[0]);
        nameL.add(typeAndName[1]);
      label_15:
      while (true) {
        switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
        case COMMA:
          ;
          break;
        default:
          jj_la1[33] = jj_gen;
          break label_15;
        }
        jj_consume_token(COMMA);
        typeAndName = FormalParameter();
          typeL.add(typeAndName[0]);
          nameL.add(typeAndName[1]);
      }
      break;
    default:
      jj_la1[34] = jj_gen;
      ;
    }
    jj_consume_token(RPAREN);
    {if (true) return new String[][] { (String[]) typeL.toArray(new String[typeL.size()]),
                            (String[]) nameL.toArray(new String[nameL.size()])};}
    throw new Error("Missing return statement in function");
  }

  final public String[] FormalParameter() throws ParseException {
  String type; String name; boolean isFinal = false;
    switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
    case FINAL:
      jj_consume_token(FINAL);
              isFinal=true;
      break;
    default:
      jj_la1[35] = jj_gen;
      ;
    }
    type = Type();
    name = VariableDeclaratorId();
    while(name.endsWith("[]")) {
      name = name.substring(0, name.length() - 2);
      type += "[]";
    }
    {if (true) return new String[] { type, name };}
    throw new Error("Missing return statement in function");
  }

  final public void ConstructorDeclaration() throws ParseException {
  Token t; String name; String[][] params; String[] exceptionTypes = null;
    switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
    case PRIVATE:
    case PROTECTED:
    case PUBLIC:
      switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
      case PUBLIC:
        jj_consume_token(PUBLIC);
                       modifiers = modifiers | Modifier.PUBLIC;
        break;
      case PROTECTED:
        jj_consume_token(PROTECTED);
                       modifiers = modifiers | Modifier.PROTECTED;
        break;
      case PRIVATE:
        jj_consume_token(PRIVATE);
                       modifiers = modifiers | Modifier.PRIVATE;
        break;
      default:
        jj_la1[36] = jj_gen;
        jj_consume_token(-1);
        throw new ParseException();
      }
      checkComment();
      break;
    default:
      jj_la1[37] = jj_gen;
      ;
    }
    t = jj_consume_token(IDENTIFIER);
    checkComment();
    params = FormalParameters();
    switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
    case THROWS:
      jj_consume_token(THROWS);
      exceptionTypes = NameList();
      break;
    default:
      jj_la1[38] = jj_gen;
      ;
    }
    sourceLocation = getSourceLocation(t);
    name = t.image;
    t = getToken(1);

    for(int i = 0; i < params[0].length; i++)
      params[0][i] = resolveClassName(params[0][i]);
    if(exceptionTypes == null) exceptionTypes = new String[0];
    for(int i = 0; i < exceptionTypes.length; i++)
      exceptionTypes[i] = resolveClassName(exceptionTypes[i]);

    ConstructorInfo ci = new ConstructorInfo(getDeclaringClass(), modifiers,
                                             name, params[0], params[1],
                                             exceptionTypes, comment);
    ci.setSourceLocation(sourceLocation);
    if(oneLineComments) ci.setComment(ci.getOneLineComment());
    comment = null;
    modifiers = 0;

    if(symbolPath.get(0) instanceof ClassInfo) {
      if(generatePrivateClassInfo ||
         (ci.getModifiers() & Modifier.PUBLIC) != 0 ||
         (ci.getModifiers() & Modifier.PROTECTED) != 0) {
        ((ClassInfo)symbolPath.get(0)).add(ci);
      }
    }
    symbolPath.add(0, ci);

    newSymbolTable.addSymbol(ci);
    enterBlock(t);
    newSymbolTable.setSymbol(ci);

    for(int i = 0; i < params[0].length; i++) {
      FieldInfo fi = new FieldInfo(getDeclaringClass(), 0,
                                   params[0][i], params[1][i], null);
      newSymbolTable.addSymbol(fi);
    }
    jj_consume_token(LBRACE);
    if (isConstructorInvocation()) {
      ExplicitConstructorInvocation();
    } else {
      ;
    }
    label_16:
    while (true) {
      switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
      case BOOLEAN:
      case BREAK:
      case BYTE:
      case CHAR:
      case CLASS:
      case CONTINUE:
      case DO:
      case DOUBLE:
      case FALSE:
      case FINAL:
      case FLOAT:
      case FOR:
      case IF:
      case INT:
      case LONG:
      case NEW:
      case NULL:
      case RETURN:
      case SHORT:
      case SUPER:
      case SWITCH:
      case SYNCHRONIZED:
      case THIS:
      case THROW:
      case TRUE:
      case TRY:
      case VOID:
      case WHILE:
      case INTEGER_LITERAL:
      case FLOATING_POINT_LITERAL:
      case CHARACTER_LITERAL:
      case STRING_LITERAL:
      case IDENTIFIER:
      case LPAREN:
      case LBRACE:
      case SEMICOLON:
      case INCR:
      case DECR:
        ;
        break;
      default:
        jj_la1[39] = jj_gen;
        break label_16;
      }
      BlockStatement();
    }
    jj_consume_token(RBRACE);
    leaveBlock(getToken(0));

    symbolPath.remove(0);
  }

  final public void ExplicitConstructorInvocation() throws ParseException {
    if (getToken(1).kind == THIS && getToken(2).kind == LPAREN) {
      jj_consume_token(THIS);
      Arguments();
      jj_consume_token(SEMICOLON);
    } else {
      switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
      case BOOLEAN:
      case BYTE:
      case CHAR:
      case DOUBLE:
      case FALSE:
      case FLOAT:
      case INT:
      case LONG:
      case NEW:
      case NULL:
      case SHORT:
      case SUPER:
      case THIS:
      case TRUE:
      case VOID:
      case INTEGER_LITERAL:
      case FLOATING_POINT_LITERAL:
      case CHARACTER_LITERAL:
      case STRING_LITERAL:
      case IDENTIFIER:
      case LPAREN:
        if (getToken(1).kind != SUPER || getToken(2).kind != LPAREN) {
          PrimaryExpression();
          jj_consume_token(DOT);
        } else {
          ;
        }
        jj_consume_token(SUPER);
        Arguments();
        jj_consume_token(SEMICOLON);
        break;
      default:
        jj_la1[40] = jj_gen;
        jj_consume_token(-1);
        throw new ParseException();
      }
    }
  }

  final public void Initializer() throws ParseException {
  Token t;
    switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
    case STATIC:
      jj_consume_token(STATIC);
      break;
    default:
      jj_la1[41] = jj_gen;
      ;
    }
    enterBlock(getToken(1));
    Block();
    leaveBlock(getToken(0));
  }

/*
 * Type, name and expression syntax follows.
 */
  final public String Type() throws ParseException {
  String type;
    switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
    case BOOLEAN:
    case BYTE:
    case CHAR:
    case DOUBLE:
    case FLOAT:
    case INT:
    case LONG:
    case SHORT:
      type = PrimitiveType();
      break;
    case IDENTIFIER:
      type = Name();
      break;
    default:
      jj_la1[42] = jj_gen;
      jj_consume_token(-1);
      throw new ParseException();
    }
    label_17:
    while (true) {
      switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
      case LBRACKET:
        ;
        break;
      default:
        jj_la1[43] = jj_gen;
        break label_17;
      }
      jj_consume_token(LBRACKET);
      jj_consume_token(RBRACKET);
                                                     type += "[]";
    }
    {if (true) return type;}
    throw new Error("Missing return statement in function");
  }

  final public String PrimitiveType() throws ParseException {
    switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
    case BOOLEAN:
      jj_consume_token(BOOLEAN);
      break;
    case CHAR:
      jj_consume_token(CHAR);
      break;
    case BYTE:
      jj_consume_token(BYTE);
      break;
    case SHORT:
      jj_consume_token(SHORT);
      break;
    case INT:
      jj_consume_token(INT);
      break;
    case LONG:
      jj_consume_token(LONG);
      break;
    case FLOAT:
      jj_consume_token(FLOAT);
      break;
    case DOUBLE:
      jj_consume_token(DOUBLE);
      break;
    default:
      jj_la1[44] = jj_gen;
      jj_consume_token(-1);
      throw new ParseException();
    }
    {if (true) return getToken(0).image;}
    throw new Error("Missing return statement in function");
  }

  final public String ResultType() throws ParseException {
  String name;
    switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
    case VOID:
      jj_consume_token(VOID);
              name = "void";
      break;
    case BOOLEAN:
    case BYTE:
    case CHAR:
    case DOUBLE:
    case FLOAT:
    case INT:
    case LONG:
    case SHORT:
    case IDENTIFIER:
      name = Type();
      break;
    default:
      jj_la1[45] = jj_gen;
      jj_consume_token(-1);
      throw new ParseException();
    }
    {if (true) return name;}
    throw new Error("Missing return statement in function");
  }

  final public String Name() throws ParseException {
  Token t; String s;
    t = jj_consume_token(IDENTIFIER);
    s = t.image;
    label_18:
    while (true) {
      if (getToken(1).kind == DOT && getToken(2).kind == IDENTIFIER) {
        ;
      } else {
        break label_18;
      }
      jj_consume_token(DOT);
      t = jj_consume_token(IDENTIFIER);
                         s += "." + t.image;
    }
    {if (true) return s;}
    throw new Error("Missing return statement in function");
  }

  final public String[] NameList() throws ParseException {
  List nl = new LinkedList(); String[] sa; String name;
    name = Name();
                nl.add(name);
    label_19:
    while (true) {
      switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
      case COMMA:
        ;
        break;
      default:
        jj_la1[46] = jj_gen;
        break label_19;
      }
      jj_consume_token(COMMA);
      name = Name();
                      nl.add(name);
    }
    {if (true) return (String[]) nl.toArray(new String[nl.size()]);}
    throw new Error("Missing return statement in function");
  }

/*
 * Expression syntax follows.
 */
  final public String Expression() throws ParseException {
  String type;
    type = ConditionalExpression();
    switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
    case ASSIGN:
    case PLUSASSIGN:
    case MINUSASSIGN:
    case STARASSIGN:
    case SLASHASSIGN:
    case ANDASSIGN:
    case ORASSIGN:
    case XORASSIGN:
    case REMASSIGN:
    case LSHIFTASSIGN:
    case RSIGNEDSHIFTASSIGN:
    case RUNSIGNEDSHIFTASSIGN:
      AssignmentOperator();
      Expression();
      break;
    default:
      jj_la1[47] = jj_gen;
      ;
    }
    {if (true) return type;}
    throw new Error("Missing return statement in function");
  }

  final public void AssignmentOperator() throws ParseException {
    switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
    case ASSIGN:
      jj_consume_token(ASSIGN);
      break;
    case STARASSIGN:
      jj_consume_token(STARASSIGN);
      break;
    case SLASHASSIGN:
      jj_consume_token(SLASHASSIGN);
      break;
    case REMASSIGN:
      jj_consume_token(REMASSIGN);
      break;
    case PLUSASSIGN:
      jj_consume_token(PLUSASSIGN);
      break;
    case MINUSASSIGN:
      jj_consume_token(MINUSASSIGN);
      break;
    case LSHIFTASSIGN:
      jj_consume_token(LSHIFTASSIGN);
      break;
    case RSIGNEDSHIFTASSIGN:
      jj_consume_token(RSIGNEDSHIFTASSIGN);
      break;
    case RUNSIGNEDSHIFTASSIGN:
      jj_consume_token(RUNSIGNEDSHIFTASSIGN);
      break;
    case ANDASSIGN:
      jj_consume_token(ANDASSIGN);
      break;
    case XORASSIGN:
      jj_consume_token(XORASSIGN);
      break;
    case ORASSIGN:
      jj_consume_token(ORASSIGN);
      break;
    default:
      jj_la1[48] = jj_gen;
      jj_consume_token(-1);
      throw new ParseException();
    }
  }

  final public String ConditionalExpression() throws ParseException {
  String type1=null, type2=null, type3=null;
  Token t1=null, t2=null, t3=null, t4=null, t5=null, t6=null;
    t1 = getToken(1);
    type1 = ConditionalOrExpression();
    t2 = getToken(0);
    switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
    case HOOK:
      jj_consume_token(HOOK);
      t3 = getToken(1);
      type2 = Expression();
      t4 = getToken(0);
      jj_consume_token(COLON);
      t5 = getToken(1);
      type3 = ConditionalExpression();
      t6 = getToken(0);
      break;
    default:
      jj_la1[49] = jj_gen;
      ;
    }
    if(type2 == null) {
      {if (true) return type1;}
    }
    else {
      if(generateSemanticErrorInfo) {
        if(!type1.equals("") && !type1.equals("boolean")) {
          error(t1, t2, "Conditional expression must be boolean type.");
        }
        if(!type2.equals("") && !type3.equals("") &&
           ((isNumericType(type2) && !isNumericType(type3)) ||
            (type2.equals("boolean") && !type3.equals("boolean")) ||
            (!isPrimitiveType(type2) && isPrimitiveType(type3)))) {
          error(t5, t6,
                "Conditional expression result types must be the same.");
        }
      }
      if(isNumericType(type2) && isNumericType(type3))
        {if (true) return binaryPromoteLeft(type2, type3);}
      else
        {if (true) return type2.equals("")? type3 : type2;}
    }
    throw new Error("Missing return statement in function");
  }

  final public String ConditionalOrExpression() throws ParseException {
  String type1=null, type2=null;
  Token t1=null, t2=null, t3=null, t4=null;
    t1=getToken(1);
    type1 = ConditionalAndExpression();
    t2=getToken(0);
    label_20:
    while (true) {
      switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
      case SC_OR:
        ;
        break;
      default:
        jj_la1[50] = jj_gen;
        break label_20;
      }
      jj_consume_token(SC_OR);
      t3=getToken(1);
      type2 = ConditionalAndExpression();
      t4=getToken(0);
    }
    {if (true) return booleanBinaryExpressionResult(type1, t1, t2, type2, t3, t4);}
    throw new Error("Missing return statement in function");
  }

  final public String ConditionalAndExpression() throws ParseException {
  String type1=null, type2=null;
  Token t1=null, t2=null, t3=null, t4=null;
    t1=getToken(1);
    type1 = InclusiveOrExpression();
    t2=getToken(0);
    label_21:
    while (true) {
      switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
      case SC_AND:
        ;
        break;
      default:
        jj_la1[51] = jj_gen;
        break label_21;
      }
      jj_consume_token(SC_AND);
      t3=getToken(1);
      type2 = InclusiveOrExpression();
      t4=getToken(0);
    }
    {if (true) return booleanBinaryExpressionResult(type1, t1, t2, type2, t3, t4);}
    throw new Error("Missing return statement in function");
  }

  final public String InclusiveOrExpression() throws ParseException {
  String type1=null, type2=null;
  Token t1=null, t2=null, t3=null, t4=null;
    t1=getToken(1);
    type1 = ExclusiveOrExpression();
    t2=getToken(0);
    label_22:
    while (true) {
      switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
      case BIT_OR:
        ;
        break;
      default:
        jj_la1[52] = jj_gen;
        break label_22;
      }
      jj_consume_token(BIT_OR);
      t3=getToken(1);
      type2 = ExclusiveOrExpression();
      t4=getToken(0);
    }
    {if (true) return logicalBinaryExpressionResult(type1, t1, t2, type2, t3, t4);}
    throw new Error("Missing return statement in function");
  }

  final public String ExclusiveOrExpression() throws ParseException {
  String type1=null, type2=null;
  Token t1=null, t2=null, t3=null, t4=null;
    t1=getToken(1);
    type1 = AndExpression();
    t2=getToken(0);
    label_23:
    while (true) {
      switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
      case XOR:
        ;
        break;
      default:
        jj_la1[53] = jj_gen;
        break label_23;
      }
      jj_consume_token(XOR);
      t3=getToken(1);
      type2 = AndExpression();
      t4=getToken(0);
    }
    {if (true) return logicalBinaryExpressionResult(type1, t1, t2, type2, t3, t4);}
    throw new Error("Missing return statement in function");
  }

  final public String AndExpression() throws ParseException {
  String type1=null, type2=null;
  Token t1=null, t2=null, t3=null, t4=null;
    t1=getToken(1);
    type1 = EqualityExpression();
    t2=getToken(0);
    label_24:
    while (true) {
      switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
      case BIT_AND:
        ;
        break;
      default:
        jj_la1[54] = jj_gen;
        break label_24;
      }
      jj_consume_token(BIT_AND);
      t3=getToken(1);
      type2 = EqualityExpression();
      t4=getToken(0);
    }
    {if (true) return logicalBinaryExpressionResult(type1, t1, t2, type2, t3, t4);}
    throw new Error("Missing return statement in function");
  }

  final public String EqualityExpression() throws ParseException {
  String type1=null, type2=null;
  Token t1=null, t2=null, t3=null, t4=null;
    t1=getToken(1);
    type1 = InstanceOfExpression();
    t2=getToken(0);
    label_25:
    while (true) {
      switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
      case EQ:
      case NE:
        ;
        break;
      default:
        jj_la1[55] = jj_gen;
        break label_25;
      }
      switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
      case EQ:
        jj_consume_token(EQ);
        break;
      case NE:
        jj_consume_token(NE);
        break;
      default:
        jj_la1[56] = jj_gen;
        jj_consume_token(-1);
        throw new ParseException();
      }
      t3=getToken(1);
      type2 = InstanceOfExpression();
      t4=getToken(0);
    }
    {if (true) return equalityBinaryExpressionResult(type1, t1, t2, type2, t3, t4);}
    throw new Error("Missing return statement in function");
  }

  final public String InstanceOfExpression() throws ParseException {
  String type=null;
    type = RelationalExpression();
    switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
    case INSTANCEOF:
      jj_consume_token(INSTANCEOF);
      Type();
      type="boolean";
      break;
    default:
      jj_la1[57] = jj_gen;
      ;
    }
    {if (true) return type;}
    throw new Error("Missing return statement in function");
  }

  final public String RelationalExpression() throws ParseException {
  String type1=null, type2=null;
  Token t1=null, t2=null, t3=null, t4=null;
    t1=getToken(1);
    type1 = ShiftExpression();
    t2=getToken(0);
    label_26:
    while (true) {
      switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
      case GT:
      case LT:
      case LE:
      case GE:
        ;
        break;
      default:
        jj_la1[58] = jj_gen;
        break label_26;
      }
      switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
      case LT:
        jj_consume_token(LT);
        break;
      case GT:
        jj_consume_token(GT);
        break;
      case LE:
        jj_consume_token(LE);
        break;
      case GE:
        jj_consume_token(GE);
        break;
      default:
        jj_la1[59] = jj_gen;
        jj_consume_token(-1);
        throw new ParseException();
      }
      t3=getToken(1);
      type2 = ShiftExpression();
      t4=getToken(0);
    }
    {if (true) return comparisonBinaryExpressionResult(type1, t1, t2, type2, t3, t4);}
    throw new Error("Missing return statement in function");
  }

  final public String ShiftExpression() throws ParseException {
  String type1=null, type2=null;
  Token t1=null, t2=null, t3=null, t4=null;
    t1=getToken(1);
    type1 = AdditiveExpression();
    t2=getToken(0);
    label_27:
    while (true) {
      switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
      case LSHIFT:
      case RSIGNEDSHIFT:
      case RUNSIGNEDSHIFT:
        ;
        break;
      default:
        jj_la1[60] = jj_gen;
        break label_27;
      }
      switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
      case LSHIFT:
        jj_consume_token(LSHIFT);
        break;
      case RSIGNEDSHIFT:
        jj_consume_token(RSIGNEDSHIFT);
        break;
      case RUNSIGNEDSHIFT:
        jj_consume_token(RUNSIGNEDSHIFT);
        break;
      default:
        jj_la1[61] = jj_gen;
        jj_consume_token(-1);
        throw new ParseException();
      }
      t3=getToken(1);
      type2 = AdditiveExpression();
      t4=getToken(0);
    }
    {if (true) return shiftBinaryExpressionResult(type1, t1, t2, type2, t3, t4);}
    throw new Error("Missing return statement in function");
  }

  final public String AdditiveExpression() throws ParseException {
  String type1=null, type2=null;
  Token t1=null, t2=null, t3=null, t4=null;
  boolean plus = false;
    t1=getToken(1);
    type1 = MultiplicativeExpression();
    t2=getToken(0);
    label_28:
    while (true) {
      switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
      case PLUS:
      case MINUS:
        ;
        break;
      default:
        jj_la1[62] = jj_gen;
        break label_28;
      }
      switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
      case PLUS:
        jj_consume_token(PLUS);
              plus=true;
        break;
      case MINUS:
        jj_consume_token(MINUS);
        break;
      default:
        jj_la1[63] = jj_gen;
        jj_consume_token(-1);
        throw new ParseException();
      }
      t3=getToken(1);
      type2 = MultiplicativeExpression();
      t4=getToken(0);
    }
    if (plus && !(isNumericType(type1) && isNumericType(type2))) {
      {if (true) return "java.lang.String";}
    } else {
      {if (true) return mathBinaryExpressionResult(type1, t1, t2, type2, t3, t4);}
    }
    throw new Error("Missing return statement in function");
  }

  final public String MultiplicativeExpression() throws ParseException {
  String type1=null, type2=null;
  Token t1=null, t2=null, t3=null, t4=null;
    t1=getToken(1);
    type1 = UnaryExpression();
    t2=getToken(0);
    label_29:
    while (true) {
      switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
      case STAR:
      case SLASH:
      case REM:
        ;
        break;
      default:
        jj_la1[64] = jj_gen;
        break label_29;
      }
      switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
      case STAR:
        jj_consume_token(STAR);
        break;
      case SLASH:
        jj_consume_token(SLASH);
        break;
      case REM:
        jj_consume_token(REM);
        break;
      default:
        jj_la1[65] = jj_gen;
        jj_consume_token(-1);
        throw new ParseException();
      }
      t3=getToken(1);
      type2 = UnaryExpression();
      t4=getToken(0);
    }
    {if (true) return mathBinaryExpressionResult(type1, t1, t2, type2, t3, t4);}
    throw new Error("Missing return statement in function");
  }

  final public String UnaryExpression() throws ParseException {
  String type; Token t0, t1, t2;
    switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
    case PLUS:
    case MINUS:
      switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
      case PLUS:
        jj_consume_token(PLUS);
        break;
      case MINUS:
        jj_consume_token(MINUS);
        break;
      default:
        jj_la1[66] = jj_gen;
        jj_consume_token(-1);
        throw new ParseException();
      }
      t0 = getToken(0);
      t1 = getToken(1);
      type = UnaryExpression();
      t2 = getToken(0);
      if(!isNumericType(type)) {
        if(generateSemanticErrorInfo) {
          error(t1, t2, "Unary expression must be of numeric type.");
        }
        type = "";
      }
      else type = unaryPromote(type);
      break;
    case INCR:
      type = PreIncrementExpression();
      break;
    case DECR:
      type = PreDecrementExpression();
      break;
    case BOOLEAN:
    case BYTE:
    case CHAR:
    case DOUBLE:
    case FALSE:
    case FLOAT:
    case INT:
    case LONG:
    case NEW:
    case NULL:
    case SHORT:
    case SUPER:
    case THIS:
    case TRUE:
    case VOID:
    case INTEGER_LITERAL:
    case FLOATING_POINT_LITERAL:
    case CHARACTER_LITERAL:
    case STRING_LITERAL:
    case IDENTIFIER:
    case LPAREN:
    case BANG:
    case TILDE:
      type = UnaryExpressionNotPlusMinus();
      break;
    default:
      jj_la1[67] = jj_gen;
      jj_consume_token(-1);
      throw new ParseException();
    }
    {if (true) return type;}
    throw new Error("Missing return statement in function");
  }

  final public String PreIncrementExpression() throws ParseException {
  String type; Token t0, t1, t2;
    jj_consume_token(INCR);
    t0 = getToken(0);
    t1 = getToken(1);
    type = PrimaryExpression();
    t2 = getToken(0);
    type = numericUnaryExpressionResult(type, t1, t2);
    {if (true) return type;}
    throw new Error("Missing return statement in function");
  }

  final public String PreDecrementExpression() throws ParseException {
  String type; Token t0, t1, t2;
    jj_consume_token(DECR);
    t0=getToken(0);
    t1 = getToken(1);
    type = PrimaryExpression();
    t2 = getToken(0);
    type = numericUnaryExpressionResult(type, t1, t2);
    {if (true) return type;}
    throw new Error("Missing return statement in function");
  }

  final public String UnaryExpressionNotPlusMinus() throws ParseException {
  String type; Token t0=null, t1=null, t2=null;
  boolean bitComplement=false, logicComplement=false;
    switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
    case BANG:
    case TILDE:
      switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
      case TILDE:
        jj_consume_token(TILDE);
              bitComplement = true;
        break;
      case BANG:
        jj_consume_token(BANG);
              logicComplement = true;
        break;
      default:
        jj_la1[68] = jj_gen;
        jj_consume_token(-1);
        throw new ParseException();
      }
      t0 = getToken(0);
      t1 = getToken(1);
      type = UnaryExpression();
      t2 = getToken(0);
      break;
    default:
      jj_la1[69] = jj_gen;
      if (isTypeCast()) {
        type = CastExpression();
      } else {
        switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
        case BOOLEAN:
        case BYTE:
        case CHAR:
        case DOUBLE:
        case FALSE:
        case FLOAT:
        case INT:
        case LONG:
        case NEW:
        case NULL:
        case SHORT:
        case SUPER:
        case THIS:
        case TRUE:
        case VOID:
        case INTEGER_LITERAL:
        case FLOATING_POINT_LITERAL:
        case CHARACTER_LITERAL:
        case STRING_LITERAL:
        case IDENTIFIER:
        case LPAREN:
          type = PostfixExpression();
          break;
        default:
          jj_la1[70] = jj_gen;
          jj_consume_token(-1);
          throw new ParseException();
        }
      }
    }
    if(bitComplement || logicComplement) {
      if(bitComplement)
        type = integralUnaryExpressionResult(type, t1, t2);
      else if(logicComplement)
        type = booleanUnaryExpressionResult(type, t1, t2);
    }
    {if (true) return type;}
    throw new Error("Missing return statement in function");
  }

  final public String PostfixExpression() throws ParseException {
  String type; Token t1, t2, t3=null; boolean postfix=false;
    t1 = getToken(1);
    type = PrimaryExpression();
    t2 = getToken(0);
    switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
    case INCR:
    case DECR:
      switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
      case INCR:
        jj_consume_token(INCR);
        break;
      case DECR:
        jj_consume_token(DECR);
        break;
      default:
        jj_la1[71] = jj_gen;
        jj_consume_token(-1);
        throw new ParseException();
      }
      t3 = getToken(0);
      postfix = true;
      break;
    default:
      jj_la1[72] = jj_gen;
      ;
    }
    if(postfix) {
      type = numericUnaryExpressionResult(type, t1, t2);
      {if (true) return type;}
    }
    else {if (true) return type;}
    throw new Error("Missing return statement in function");
  }

  final public String CastExpression() throws ParseException {
  String fromType, toType; Token t1, t2;
    if (isPrimitiveTypeCast()) {
      jj_consume_token(LPAREN);
      t1 = getToken(0);
      toType = Type();
      t2 = getToken(1);
      jj_consume_token(RPAREN);
      fromType = UnaryExpression();
    } else {
      switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
      case LPAREN:
        jj_consume_token(LPAREN);
      t1 = getToken(0);
        toType = Type();
      t2 = getToken(1);
        jj_consume_token(RPAREN);
        fromType = UnaryExpressionNotPlusMinus();
        break;
      default:
        jj_la1[73] = jj_gen;
        jj_consume_token(-1);
        throw new ParseException();
      }
    }
    toType = resolveClassName(toType);
    if(generateSemanticErrorInfo &&
       !classTable.isAssignableTo(toType, fromType, true) &&
       !classTable.isAssignableTo(fromType, toType, true)) {
      error(t1, t2, "Cannnot cast " + fromType + " to " + toType + ".");
    }
    {if (true) return toType;}
    throw new Error("Missing return statement in function");
  }

  final public String PrimaryExpression() throws ParseException {
  String type; Token t0;
    t0=getToken(1);
    type = PrimaryPrefix();
    label_30:
    while (true) {
      if (isPrimarySuffix()) {
        ;
      } else {
        break label_30;
      }
      type = PrimarySuffix(type, t0);
    }
    {if (true) return type;}
    throw new Error("Missing return statement in function");
  }

  final public String PrimaryPrefix() throws ParseException {
  String type=""; Token[] nt; String[] args = null; Token t1, t2;
    switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
    case FALSE:
    case NULL:
    case TRUE:
    case INTEGER_LITERAL:
    case FLOATING_POINT_LITERAL:
    case CHARACTER_LITERAL:
    case STRING_LITERAL:
      t1=getToken(1);
      type = Literal();
      t2=getToken(0);
      break;
    default:
      jj_la1[75] = jj_gen;
      if (jj_2_1(1)) {
        if (isThis()) {

        } else {
          jj_consume_token(-1);
          throw new ParseException();
        }
      type=getDeclaringClass();
      t1=getToken(1);
        if (getToken(1).kind != THIS) {
          type = Name();
        type=resolveClassName(type);
          jj_consume_token(DOT);
        } else {
          ;
        }
        jj_consume_token(THIS);
      t2=getToken(0);
      } else {
        switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
        case SUPER:
          t1 = jj_consume_token(SUPER);
      // Find the superclass name
      for(Iterator i = symbolPath.iterator(); i.hasNext(); ) {
        MemberInfo s = (MemberInfo) i.next();
        if(s instanceof ClassInfo) {
          type = ((ClassInfo)s).getSuperclass();
          break;
        }
      }
          break;
        default:
          jj_la1[76] = jj_gen;
          if (jj_2_2(1)) {
            type = AllocationExpression();
          } else if (jj_2_3(1)) {
            if (isClassSelector()) {

            } else {
              jj_consume_token(-1);
              throw new ParseException();
            }
      t1=getToken(1);
            ResultType();
            jj_consume_token(DOT);
            jj_consume_token(CLASS);
      t2=getToken(0);
      type="java.lang.Class";
          } else {
            switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
            case LPAREN:
      t1=getToken(1);
              jj_consume_token(LPAREN);
              type = Expression();
              jj_consume_token(RPAREN);
      t2=getToken(0);
              break;
            case IDENTIFIER:
              nt = PrimaryName();
              switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
              case LPAREN:
                args = Arguments();
                break;
              default:
                jj_la1[74] = jj_gen;
                ;
              }
      MemberInfo mi = findPrimaryType(nt, args);
      type = (mi != null? resolveClassName(mi.getType()) : "");
      if(mi instanceof ClassInfo) {
        staticPrimary = getToken(0);
      }
              break;
            default:
              jj_la1[77] = jj_gen;
              jj_consume_token(-1);
              throw new ParseException();
            }
          }
        }
      }
    }
    {if (true) return type;}
    throw new Error("Missing return statement in function");
  }

  final public String PrimarySuffix(String prefixType, Token exprStart) throws ParseException {
  String indexType="", type=""; Token t0, t1, t2, t3; String[] args=null;
    switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
    case LBRACKET:
      jj_consume_token(LBRACKET);
      t0=getToken(0);
      t1=getToken(1);
      indexType = Expression();
      t2=getToken(0);
      t3=getToken(1);
      jj_consume_token(RBRACKET);
      if(generateSemanticErrorInfo) {
        if(!indexType.equals("") &&
           !indexType.equals("byte") && !indexType.equals("char") &&
           !indexType.equals("short") && !indexType.equals("int")) {
          if(isNumericType(indexType))
            error(t1, t2, "Incompatible type for []. Explicit cast needed " +
                          "to convert " + indexType + " to int.");
          else
            error(t1, t2, "Incompatible type for []. Can't convert " +
                          indexType + " to int.");
        }
      }

      if(!prefixType.equals("")) {
        if(!prefixType.endsWith("[]")) {
          if(generateSemanticErrorInfo) {
            error(t0, "[] can only be applied to arrays. " +
                      "It can't be applied to " + prefixType + ".");
          }
        }
        else {
          type = prefixType.substring(0, prefixType.length() - 2);
        }
      }
      break;
    case DOT:
      jj_consume_token(DOT);
      t0 = jj_consume_token(IDENTIFIER);
      switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
      case LPAREN:
        args = Arguments();
        break;
      default:
        jj_la1[78] = jj_gen;
        ;
      }
      MemberInfo mi = findPrimaryType(prefixType, prefixType,
                                      null, false, t0, args);
      type = (mi != null? resolveClassName(mi.getType()) : "");
      break;
    default:
      jj_la1[79] = jj_gen;
      jj_consume_token(-1);
      throw new ParseException();
    }
    {if (true) return type;}
    throw new Error("Missing return statement in function");
  }

  final public Token[] PrimaryName() throws ParseException {
  List tokenList=new LinkedList(); Token t;
    t = jj_consume_token(IDENTIFIER);
    tokenList.add(t);
    label_31:
    while (true) {
      if (getToken(1).kind == DOT && getToken(2).kind == IDENTIFIER) {
        ;
      } else {
        break label_31;
      }
      jj_consume_token(DOT);
      t = jj_consume_token(IDENTIFIER);
                         tokenList.add(t);
    }
    {if (true) return (Token[]) tokenList.toArray(new Token[tokenList.size()]);}
    throw new Error("Missing return statement in function");
  }

  final public String Literal() throws ParseException {
  String type;
    switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
    case INTEGER_LITERAL:
      jj_consume_token(INTEGER_LITERAL);
                               type = "int";
      break;
    case FLOATING_POINT_LITERAL:
      jj_consume_token(FLOATING_POINT_LITERAL);
                               type = "double";
      break;
    case CHARACTER_LITERAL:
      jj_consume_token(CHARACTER_LITERAL);
                               type = "char";
      break;
    case STRING_LITERAL:
      jj_consume_token(STRING_LITERAL);
                               type = "java.lang.String";
      break;
    case FALSE:
    case TRUE:
      BooleanLiteral();
                               type = "boolean";
      break;
    case NULL:
      NullLiteral();
                               type = "";
      break;
    default:
      jj_la1[80] = jj_gen;
      jj_consume_token(-1);
      throw new ParseException();
    }
    {if (true) return type;}
    throw new Error("Missing return statement in function");
  }

  final public String BooleanLiteral() throws ParseException {
  String val;
    switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
    case TRUE:
      jj_consume_token(TRUE);
             val = "true";
      break;
    case FALSE:
      jj_consume_token(FALSE);
              val = "false";
      break;
    default:
      jj_la1[81] = jj_gen;
      jj_consume_token(-1);
      throw new ParseException();
    }
    {if (true) return val;}
    throw new Error("Missing return statement in function");
  }

  final public void NullLiteral() throws ParseException {
    jj_consume_token(NULL);
  }

  final public String[] Arguments() throws ParseException {
  String[] list = new String[0];
    jj_consume_token(LPAREN);
    switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
    case BOOLEAN:
    case BYTE:
    case CHAR:
    case DOUBLE:
    case FALSE:
    case FLOAT:
    case INT:
    case LONG:
    case NEW:
    case NULL:
    case SHORT:
    case SUPER:
    case THIS:
    case TRUE:
    case VOID:
    case INTEGER_LITERAL:
    case FLOATING_POINT_LITERAL:
    case CHARACTER_LITERAL:
    case STRING_LITERAL:
    case IDENTIFIER:
    case LPAREN:
    case BANG:
    case TILDE:
    case INCR:
    case DECR:
    case PLUS:
    case MINUS:
      list = ArgumentList();
      break;
    default:
      jj_la1[82] = jj_gen;
      ;
    }
    jj_consume_token(RPAREN);
    {if (true) return list;}
    throw new Error("Missing return statement in function");
  }

  final public String[] ArgumentList() throws ParseException {
  List list = new LinkedList(); String type;
    type = Expression();
                        list.add(type);
    label_32:
    while (true) {
      switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
      case COMMA:
        ;
        break;
      default:
        jj_la1[83] = jj_gen;
        break label_32;
      }
      jj_consume_token(COMMA);
      type = Expression();
                            list.add(type);
    }
    {if (true) return (String[]) list.toArray(new String[list.size()]);}
    throw new Error("Missing return statement in function");
  }

  final public String AllocationExpression() throws ParseException {
  String type; int dims = 0;
    if (getToken(2).kind != IDENTIFIER) {
      jj_consume_token(NEW);
      type = PrimitiveType();
      dims = ArrayDimsAndInits();
    } else {
      switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
      case NEW:
        jj_consume_token(NEW);
        type = Name();
        switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
        case LBRACKET:
          dims = ArrayDimsAndInits();
          break;
        case LPAREN:
          Arguments();
          switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
          case LBRACE:
          enterBlock(getToken(1));
            ClassBody();
          leaveBlock(getToken(0));
            break;
          default:
            jj_la1[84] = jj_gen;
            ;
          }
          break;
        default:
          jj_la1[85] = jj_gen;
          jj_consume_token(-1);
          throw new ParseException();
        }
        break;
      default:
        jj_la1[86] = jj_gen;
        jj_consume_token(-1);
        throw new ParseException();
      }
    }
    while(dims > 0) { type += "[]"; dims--; }
    {if (true) return resolveClassName(type);}
    throw new Error("Missing return statement in function");
  }

/*
 * The third LOOKAHEAD specification below is to parse to PrimarySuffix
 * if there is an expression between the "[...]".
 */
  final public int ArrayDimsAndInits() throws ParseException {
  int dims = 0;
    if (getToken(2).kind != RBRACKET) {
      label_33:
      while (true) {
        jj_consume_token(LBRACKET);
        Expression();
        jj_consume_token(RBRACKET);
                           dims++;
        if (getToken(1).kind == LBRACKET && getToken(2).kind != RBRACKET) {
          ;
        } else {
          break label_33;
        }
      }
      label_34:
      while (true) {
        if (getToken(1).kind == LBRACKET && getToken(2).kind == RBRACKET) {
          ;
        } else {
          break label_34;
        }
        jj_consume_token(LBRACKET);
        jj_consume_token(RBRACKET);
              dims++;
      }
    } else {
      switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
      case LBRACKET:
        label_35:
        while (true) {
          jj_consume_token(LBRACKET);
          jj_consume_token(RBRACKET);
              dims++;
          switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
          case LBRACKET:
            ;
            break;
          default:
            jj_la1[87] = jj_gen;
            break label_35;
          }
        }
        ArrayInitializer();
        break;
      default:
        jj_la1[88] = jj_gen;
        jj_consume_token(-1);
        throw new ParseException();
      }
    }
    {if (true) return dims;}
    throw new Error("Missing return statement in function");
  }

/*
 * Statement syntax follows.
 */
  final public void Statement() throws ParseException {
    if (getToken(1).kind == IDENTIFIER && getToken(2).kind == COLON) {
      LabeledStatement();
    } else {
      switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
      case LBRACE:
        Block();
        break;
      case SEMICOLON:
        EmptyStatement();
        break;
      case BOOLEAN:
      case BYTE:
      case CHAR:
      case DOUBLE:
      case FALSE:
      case FLOAT:
      case INT:
      case LONG:
      case NEW:
      case NULL:
      case SHORT:
      case SUPER:
      case THIS:
      case TRUE:
      case VOID:
      case INTEGER_LITERAL:
      case FLOATING_POINT_LITERAL:
      case CHARACTER_LITERAL:
      case STRING_LITERAL:
      case IDENTIFIER:
      case LPAREN:
      case INCR:
      case DECR:
        try {
          StatementExpression();
          jj_consume_token(SEMICOLON);
        } catch (ParseException pex) {
    if(generateSyntaxErrorInfo) {
      error(pex.currentToken.next, "Syntax error.");
    }
    // This will safely get us out of most errors.
    ErrorSkipToEndOfStatement();
        }
        break;
      case SWITCH:
        SwitchStatement();
        break;
      case IF:
        IfStatement();
        break;
      case WHILE:
        WhileStatement();
        break;
      case DO:
        DoStatement();
        break;
      case FOR:
        ForStatement();
        break;
      case BREAK:
        BreakStatement();
        break;
      case CONTINUE:
        ContinueStatement();
        break;
      case RETURN:
        ReturnStatement();
        break;
      case THROW:
        ThrowStatement();
        break;
      case SYNCHRONIZED:
        SynchronizedStatement();
        break;
      case TRY:
        TryStatement();
        break;
      default:
        jj_la1[89] = jj_gen;
        jj_consume_token(-1);
        throw new ParseException();
      }
    }
  }

  final public void LabeledStatement() throws ParseException {
    jj_consume_token(IDENTIFIER);
    jj_consume_token(COLON);
    Statement();
  }

  final public void Block() throws ParseException {
  Token t;
    jj_consume_token(LBRACE);
      enterBlock(getToken(0));
    label_36:
    while (true) {
      switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
      case BOOLEAN:
      case BREAK:
      case BYTE:
      case CHAR:
      case CLASS:
      case CONTINUE:
      case DO:
      case DOUBLE:
      case FALSE:
      case FINAL:
      case FLOAT:
      case FOR:
      case IF:
      case INT:
      case LONG:
      case NEW:
      case NULL:
      case RETURN:
      case SHORT:
      case SUPER:
      case SWITCH:
      case SYNCHRONIZED:
      case THIS:
      case THROW:
      case TRUE:
      case TRY:
      case VOID:
      case WHILE:
      case INTEGER_LITERAL:
      case FLOATING_POINT_LITERAL:
      case CHARACTER_LITERAL:
      case STRING_LITERAL:
      case IDENTIFIER:
      case LPAREN:
      case LBRACE:
      case SEMICOLON:
      case INCR:
      case DECR:
        ;
        break;
      default:
        jj_la1[90] = jj_gen;
        break label_36;
      }
      BlockStatement();
    }
      leaveBlock(getToken(1));
    jj_consume_token(RBRACE);
  }

  final public void BlockStatement() throws ParseException {
    if (isLocalVariableDeclaration()) {
      LocalVariableDeclaration();
      jj_consume_token(SEMICOLON);
    } else {
      switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
      case BOOLEAN:
      case BREAK:
      case BYTE:
      case CHAR:
      case CONTINUE:
      case DO:
      case DOUBLE:
      case FALSE:
      case FLOAT:
      case FOR:
      case IF:
      case INT:
      case LONG:
      case NEW:
      case NULL:
      case RETURN:
      case SHORT:
      case SUPER:
      case SWITCH:
      case SYNCHRONIZED:
      case THIS:
      case THROW:
      case TRUE:
      case TRY:
      case VOID:
      case WHILE:
      case INTEGER_LITERAL:
      case FLOATING_POINT_LITERAL:
      case CHARACTER_LITERAL:
      case STRING_LITERAL:
      case IDENTIFIER:
      case LPAREN:
      case LBRACE:
      case SEMICOLON:
      case INCR:
      case DECR:
        Statement();
        break;
      case CLASS:
        UnmodifiedClassDeclaration();
        break;
      case INTERFACE:
        UnmodifiedInterfaceDeclaration();
        break;
      default:
        jj_la1[91] = jj_gen;
        jj_consume_token(-1);
        throw new ParseException();
      }
    }
  }

  final public void LocalVariableDeclaration() throws ParseException {
  String type; String thisType; String name; FieldInfo fi;
    switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
    case FINAL:
      jj_consume_token(FINAL);
              modifiers = modifiers | Modifier.FINAL;
      break;
    default:
      jj_la1[92] = jj_gen;
      ;
    }
    type = Type();
    name = VariableDeclarator();
    thisType = type;
    while(name.endsWith("[]")) {
      name = name.substring(0, name.length() - 2);
      thisType += "[]";
    }
    thisType = resolveClassName(thisType);
    fi = new FieldInfo(getDeclaringClass(), modifiers,
                       thisType, name, comment);
    fi.setSourceLocation(sourceLocation);
    if(oneLineComments) fi.setComment(fi.getOneLineComment());
    newSymbolTable.addSymbol(fi);
    label_37:
    while (true) {
      switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
      case COMMA:
        ;
        break;
      default:
        jj_la1[93] = jj_gen;
        break label_37;
      }
      jj_consume_token(COMMA);
      name = VariableDeclarator();
      thisType = type;
      while(name.endsWith("[]")) {
        name = name.substring(0, name.length() - 2);
        thisType += "[]";
      }
      thisType = resolveClassName(thisType);
      fi = new FieldInfo(getDeclaringClass(), modifiers, thisType,
                         name, comment);
      fi.setSourceLocation(sourceLocation);
      if(oneLineComments) fi.setComment(fi.getOneLineComment());
      newSymbolTable.addSymbol(fi);
    }
    comment = null;
    modifiers = 0;
  }

  final public void EmptyStatement() throws ParseException {
    jj_consume_token(SEMICOLON);
  }

  final public void StatementExpression() throws ParseException {
    switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
    case INCR:
      PreIncrementExpression();
      break;
    case DECR:
      PreDecrementExpression();
      break;
    case BOOLEAN:
    case BYTE:
    case CHAR:
    case DOUBLE:
    case FALSE:
    case FLOAT:
    case INT:
    case LONG:
    case NEW:
    case NULL:
    case SHORT:
    case SUPER:
    case THIS:
    case TRUE:
    case VOID:
    case INTEGER_LITERAL:
    case FLOATING_POINT_LITERAL:
    case CHARACTER_LITERAL:
    case STRING_LITERAL:
    case IDENTIFIER:
    case LPAREN:
      PrimaryExpression();
      switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
      case ASSIGN:
      case INCR:
      case DECR:
      case PLUSASSIGN:
      case MINUSASSIGN:
      case STARASSIGN:
      case SLASHASSIGN:
      case ANDASSIGN:
      case ORASSIGN:
      case XORASSIGN:
      case REMASSIGN:
      case LSHIFTASSIGN:
      case RSIGNEDSHIFTASSIGN:
      case RUNSIGNEDSHIFTASSIGN:
        switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
        case INCR:
          jj_consume_token(INCR);
          break;
        case DECR:
          jj_consume_token(DECR);
          break;
        case ASSIGN:
        case PLUSASSIGN:
        case MINUSASSIGN:
        case STARASSIGN:
        case SLASHASSIGN:
        case ANDASSIGN:
        case ORASSIGN:
        case XORASSIGN:
        case REMASSIGN:
        case LSHIFTASSIGN:
        case RSIGNEDSHIFTASSIGN:
        case RUNSIGNEDSHIFTASSIGN:
          AssignmentOperator();
          Expression();
          break;
        default:
          jj_la1[94] = jj_gen;
          jj_consume_token(-1);
          throw new ParseException();
        }
        break;
      default:
        jj_la1[95] = jj_gen;
        ;
      }
      break;
    default:
      jj_la1[96] = jj_gen;
      jj_consume_token(-1);
      throw new ParseException();
    }
  }

  final public void SwitchStatement() throws ParseException {
  Token t;
    jj_consume_token(SWITCH);
    jj_consume_token(LPAREN);
    Expression();
    jj_consume_token(RPAREN);
    jj_consume_token(LBRACE);
      enterBlock(getToken(0));
    label_38:
    while (true) {
      switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
      case CASE:
      case _DEFAULT:
        ;
        break;
      default:
        jj_la1[97] = jj_gen;
        break label_38;
      }
      SwitchLabel();
      label_39:
      while (true) {
        switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
        case BOOLEAN:
        case BREAK:
        case BYTE:
        case CHAR:
        case CLASS:
        case CONTINUE:
        case DO:
        case DOUBLE:
        case FALSE:
        case FINAL:
        case FLOAT:
        case FOR:
        case IF:
        case INT:
        case LONG:
        case NEW:
        case NULL:
        case RETURN:
        case SHORT:
        case SUPER:
        case SWITCH:
        case SYNCHRONIZED:
        case THIS:
        case THROW:
        case TRUE:
        case TRY:
        case VOID:
        case WHILE:
        case INTEGER_LITERAL:
        case FLOATING_POINT_LITERAL:
        case CHARACTER_LITERAL:
        case STRING_LITERAL:
        case IDENTIFIER:
        case LPAREN:
        case LBRACE:
        case SEMICOLON:
        case INCR:
        case DECR:
          ;
          break;
        default:
          jj_la1[98] = jj_gen;
          break label_39;
        }
        BlockStatement();
      }
    }
      leaveBlock(getToken(1));
    jj_consume_token(RBRACE);
  }

  final public void SwitchLabel() throws ParseException {
    switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
    case CASE:
      jj_consume_token(CASE);
      Expression();
      jj_consume_token(COLON);
      break;
    case _DEFAULT:
      jj_consume_token(_DEFAULT);
      jj_consume_token(COLON);
      break;
    default:
      jj_la1[99] = jj_gen;
      jj_consume_token(-1);
      throw new ParseException();
    }
  }

  final public void IfStatement() throws ParseException {
    jj_consume_token(IF);
    jj_consume_token(LPAREN);
    Expression();
    jj_consume_token(RPAREN);
    Statement();
    switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
    case ELSE:
      jj_consume_token(ELSE);
      Statement();
      break;
    default:
      jj_la1[100] = jj_gen;
      ;
    }
  }

  final public void WhileStatement() throws ParseException {
    jj_consume_token(WHILE);
    jj_consume_token(LPAREN);
    Expression();
    jj_consume_token(RPAREN);
    Statement();
  }

  final public void DoStatement() throws ParseException {
    jj_consume_token(DO);
    Statement();
    jj_consume_token(WHILE);
    jj_consume_token(LPAREN);
    Expression();
    jj_consume_token(RPAREN);
    jj_consume_token(SEMICOLON);
  }

  final public void ForStatement() throws ParseException {
  Token t;
    jj_consume_token(FOR);
    enterBlock(getToken(0));
    jj_consume_token(LPAREN);
    switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
    case BOOLEAN:
    case BYTE:
    case CHAR:
    case DOUBLE:
    case FALSE:
    case FINAL:
    case FLOAT:
    case INT:
    case LONG:
    case NEW:
    case NULL:
    case SHORT:
    case SUPER:
    case THIS:
    case TRUE:
    case VOID:
    case INTEGER_LITERAL:
    case FLOATING_POINT_LITERAL:
    case CHARACTER_LITERAL:
    case STRING_LITERAL:
    case IDENTIFIER:
    case LPAREN:
    case INCR:
    case DECR:
      ForInit();
      break;
    default:
      jj_la1[101] = jj_gen;
      ;
    }
    jj_consume_token(SEMICOLON);
    switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
    case BOOLEAN:
    case BYTE:
    case CHAR:
    case DOUBLE:
    case FALSE:
    case FLOAT:
    case INT:
    case LONG:
    case NEW:
    case NULL:
    case SHORT:
    case SUPER:
    case THIS:
    case TRUE:
    case VOID:
    case INTEGER_LITERAL:
    case FLOATING_POINT_LITERAL:
    case CHARACTER_LITERAL:
    case STRING_LITERAL:
    case IDENTIFIER:
    case LPAREN:
    case BANG:
    case TILDE:
    case INCR:
    case DECR:
    case PLUS:
    case MINUS:
      Expression();
      break;
    default:
      jj_la1[102] = jj_gen;
      ;
    }
    jj_consume_token(SEMICOLON);
    switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
    case BOOLEAN:
    case BYTE:
    case CHAR:
    case DOUBLE:
    case FALSE:
    case FLOAT:
    case INT:
    case LONG:
    case NEW:
    case NULL:
    case SHORT:
    case SUPER:
    case THIS:
    case TRUE:
    case VOID:
    case INTEGER_LITERAL:
    case FLOATING_POINT_LITERAL:
    case CHARACTER_LITERAL:
    case STRING_LITERAL:
    case IDENTIFIER:
    case LPAREN:
    case INCR:
    case DECR:
      ForUpdate();
      break;
    default:
      jj_la1[103] = jj_gen;
      ;
    }
    jj_consume_token(RPAREN);
    Statement();
    leaveBlock(getToken(0));
  }

  final public void ForInit() throws ParseException {
    if (isLocalVariableDeclaration()) {
      LocalVariableDeclaration();
    } else {
      switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
      case BOOLEAN:
      case BYTE:
      case CHAR:
      case DOUBLE:
      case FALSE:
      case FLOAT:
      case INT:
      case LONG:
      case NEW:
      case NULL:
      case SHORT:
      case SUPER:
      case THIS:
      case TRUE:
      case VOID:
      case INTEGER_LITERAL:
      case FLOATING_POINT_LITERAL:
      case CHARACTER_LITERAL:
      case STRING_LITERAL:
      case IDENTIFIER:
      case LPAREN:
      case INCR:
      case DECR:
        StatementExpressionList();
        break;
      default:
        jj_la1[104] = jj_gen;
        jj_consume_token(-1);
        throw new ParseException();
      }
    }
  }

  final public void StatementExpressionList() throws ParseException {
    StatementExpression();
    label_40:
    while (true) {
      switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
      case COMMA:
        ;
        break;
      default:
        jj_la1[105] = jj_gen;
        break label_40;
      }
      jj_consume_token(COMMA);
      StatementExpression();
    }
  }

  final public void ForUpdate() throws ParseException {
    StatementExpressionList();
  }

  final public void BreakStatement() throws ParseException {
    jj_consume_token(BREAK);
    switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
    case IDENTIFIER:
      jj_consume_token(IDENTIFIER);
      break;
    default:
      jj_la1[106] = jj_gen;
      ;
    }
    jj_consume_token(SEMICOLON);
  }

  final public void ContinueStatement() throws ParseException {
    jj_consume_token(CONTINUE);
    switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
    case IDENTIFIER:
      jj_consume_token(IDENTIFIER);
      break;
    default:
      jj_la1[107] = jj_gen;
      ;
    }
    jj_consume_token(SEMICOLON);
  }

  final public void ReturnStatement() throws ParseException {
    jj_consume_token(RETURN);
    switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
    case BOOLEAN:
    case BYTE:
    case CHAR:
    case DOUBLE:
    case FALSE:
    case FLOAT:
    case INT:
    case LONG:
    case NEW:
    case NULL:
    case SHORT:
    case SUPER:
    case THIS:
    case TRUE:
    case VOID:
    case INTEGER_LITERAL:
    case FLOATING_POINT_LITERAL:
    case CHARACTER_LITERAL:
    case STRING_LITERAL:
    case IDENTIFIER:
    case LPAREN:
    case BANG:
    case TILDE:
    case INCR:
    case DECR:
    case PLUS:
    case MINUS:
      Expression();
      break;
    default:
      jj_la1[108] = jj_gen;
      ;
    }
    jj_consume_token(SEMICOLON);
  }

  final public void ThrowStatement() throws ParseException {
    jj_consume_token(THROW);
    Expression();
    jj_consume_token(SEMICOLON);
  }

  final public void SynchronizedStatement() throws ParseException {
    jj_consume_token(SYNCHRONIZED);
    jj_consume_token(LPAREN);
    Expression();
    jj_consume_token(RPAREN);
    Block();
  }

  final public void TryStatement() throws ParseException {
  String[] typeAndName;
    jj_consume_token(TRY);
    Block();
    label_41:
    while (true) {
      switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
      case CATCH:
        ;
        break;
      default:
        jj_la1[109] = jj_gen;
        break label_41;
      }
      enterBlock(getToken(1));
      jj_consume_token(CATCH);
      jj_consume_token(LPAREN);
      typeAndName = FormalParameter();
      FieldInfo fi = new FieldInfo(getDeclaringClass(), 0,
                                   resolveClassName(typeAndName[0]),
                                   typeAndName[1], null);
      newSymbolTable.addSymbol(fi);
      jj_consume_token(RPAREN);
      Block();
      leaveBlock(getToken(0));
    }
    switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
    case FINALLY:
      jj_consume_token(FINALLY);
      Block();
      break;
    default:
      jj_la1[110] = jj_gen;
      ;
    }
  }

  void ErrorSkipToEndOfStatement() throws ParseException {
  int braceLevel = 1;

  while(true) {
    switch(getToken(1).kind) {
      case LBRACE:    braceLevel++; break;
      case RBRACE:    braceLevel--; break;

      // If there is a semicolon that is not inside other braces, then
      // it should be safe to continue parsing.
      case SEMICOLON: if(braceLevel == 1) braceLevel = 0; break;
    }
    if(braceLevel == 0) break;
    else getNextToken();
  }
  }

  final private boolean jj_2_1(int xla) {
    jj_la = xla; jj_lastpos = jj_scanpos = token;
    boolean retval = !jj_3_1();
    jj_save(0, xla);
    return retval;
  }

  final private boolean jj_2_2(int xla) {
    jj_la = xla; jj_lastpos = jj_scanpos = token;
    boolean retval = !jj_3_2();
    jj_save(1, xla);
    return retval;
  }

  final private boolean jj_2_3(int xla) {
    jj_la = xla; jj_lastpos = jj_scanpos = token;
    boolean retval = !jj_3_3();
    jj_save(2, xla);
    return retval;
  }

  final private boolean jj_3R_44() {
    Token xsp;
    xsp = jj_scanpos;
    lookingAhead = true;
    jj_semLA = getToken(2).kind != IDENTIFIER;
    lookingAhead = false;
    if (!jj_semLA || jj_3R_48()) {
    jj_scanpos = xsp;
    if (jj_3R_49()) return true;
    if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
    } else if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
    return false;
  }

  final private boolean jj_3R_58() {
    if (jj_scan_token(BYTE)) return true;
    if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
    return false;
  }

  final private boolean jj_3R_57() {
    if (jj_scan_token(CHAR)) return true;
    if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
    return false;
  }

  final private boolean jj_3R_56() {
    if (jj_scan_token(BOOLEAN)) return true;
    if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
    return false;
  }

  final private boolean jj_3R_55() {
    Token xsp;
    xsp = jj_scanpos;
    if (jj_3R_56()) {
    jj_scanpos = xsp;
    if (jj_3R_57()) {
    jj_scanpos = xsp;
    if (jj_3R_58()) {
    jj_scanpos = xsp;
    if (jj_3R_59()) {
    jj_scanpos = xsp;
    if (jj_3R_60()) {
    jj_scanpos = xsp;
    if (jj_3R_61()) {
    jj_scanpos = xsp;
    if (jj_3R_62()) {
    jj_scanpos = xsp;
    if (jj_3R_63()) return true;
    if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
    } else if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
    } else if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
    } else if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
    } else if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
    } else if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
    } else if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
    } else if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
    return false;
  }

  final private boolean jj_3R_53() {
    if (jj_3R_55()) return true;
    if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
    return false;
  }

  final private boolean jj_3R_52() {
    Token xsp;
    xsp = jj_scanpos;
    if (jj_3R_53()) {
    jj_scanpos = xsp;
    if (jj_3R_54()) return true;
    if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
    } else if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
    return false;
  }

  final private boolean jj_3R_45() {
    return false;
  }

  final private boolean jj_3R_51() {
    if (jj_3R_52()) return true;
    if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
    return false;
  }

  final private boolean jj_3_3() {
    Token xsp;
    xsp = jj_scanpos;
    lookingAhead = true;
    jj_semLA = isClassSelector();
    lookingAhead = false;
    if (!jj_semLA || jj_3R_45()) return true;
    if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
    if (jj_3R_46()) return true;
    if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
    return false;
  }

  final private boolean jj_3_2() {
    if (jj_3R_44()) return true;
    if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
    return false;
  }

  final private boolean jj_3R_42() {
    return false;
  }

  final private boolean jj_3R_47() {
    if (jj_scan_token(IDENTIFIER)) return true;
    if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
    return false;
  }

  final private boolean jj_3R_43() {
    if (jj_3R_47()) return true;
    if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
    return false;
  }

  final private boolean jj_3R_50() {
    if (jj_scan_token(VOID)) return true;
    if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
    return false;
  }

  final private boolean jj_3R_46() {
    Token xsp;
    xsp = jj_scanpos;
    if (jj_3R_50()) {
    jj_scanpos = xsp;
    if (jj_3R_51()) return true;
    if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
    } else if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
    return false;
  }

  final private boolean jj_3R_54() {
    if (jj_3R_47()) return true;
    if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
    return false;
  }

  final private boolean jj_3_1() {
    Token xsp;
    xsp = jj_scanpos;
    lookingAhead = true;
    jj_semLA = isThis();
    lookingAhead = false;
    if (!jj_semLA || jj_3R_42()) return true;
    if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
    xsp = jj_scanpos;
    if (jj_3R_43()) jj_scanpos = xsp;
    else if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
    if (jj_scan_token(THIS)) return true;
    if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
    return false;
  }

  final private boolean jj_3R_63() {
    if (jj_scan_token(DOUBLE)) return true;
    if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
    return false;
  }

  final private boolean jj_3R_62() {
    if (jj_scan_token(FLOAT)) return true;
    if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
    return false;
  }

  final private boolean jj_3R_49() {
    if (jj_scan_token(NEW)) return true;
    if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
    return false;
  }

  final private boolean jj_3R_61() {
    if (jj_scan_token(LONG)) return true;
    if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
    return false;
  }

  final private boolean jj_3R_60() {
    if (jj_scan_token(INT)) return true;
    if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
    return false;
  }

  final private boolean jj_3R_48() {
    if (jj_scan_token(NEW)) return true;
    if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
    return false;
  }

  final private boolean jj_3R_59() {
    if (jj_scan_token(SHORT)) return true;
    if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
    return false;
  }

  public JavaParserTokenManager token_source;
  ASCII_UCodeESC_CharStream jj_input_stream;
  public Token token, jj_nt;
  private int jj_ntk;
  private Token jj_scanpos, jj_lastpos;
  private int jj_la;
  public boolean lookingAhead = false;
  private boolean jj_semLA;
  private int jj_gen;
  final private int[] jj_la1 = new int[111];
  final private int[] jj_la1_0 = {0x0,0x0,0x20102000,0x0,0x2000,0x20002000,0x20002000,0x8000000,0x0,0xa2196000,0x20002000,0x20002000,0xa2094000,0x2000,0x2000,0x20002000,0x20002000,0x8000000,0xa2196000,0xa2094000,0x20000000,0x20000000,0x0,0x0,0x0,0x92094000,0x92094000,0x0,0x20002000,0x20002000,0x0,0x0,0x0,0x0,0xa2094000,0x20000000,0x0,0x0,0x0,0xb359c000,0x92094000,0x0,0x82094000,0x0,0x82094000,0x82094000,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x92094000,0x0,0x0,0x92094000,0x0,0x0,0x0,0x0,0x10000000,0x0,0x0,0x0,0x0,0x10000000,0x10000000,0x92094000,0x0,0x0,0x0,0x0,0x0,0x0,0x9349c000,0xb359c000,0x9359c000,0x20000000,0x0,0x0,0x0,0x92094000,0x820000,0xb359c000,0x820000,0x4000000,0xb2094000,0x92094000,0x92094000,0x92094000,0x0,0x0,0x0,0x92094000,0x40000,0x40000000,};
  final private int[] jj_la1_1 = {0x1000,0x10,0x8080,0x0,0x8080,0x8000,0x8000,0x0,0x8,0x644ee3c0,0x4e000,0x4e000,0x4406e140,0x8000,0x8000,0x4e000,0x4e000,0x0,0x644ee3c0,0x4406e140,0x4404e000,0x4404e000,0x0,0x0,0x0,0x28920d40,0x28920d40,0x0,0x4ce200,0x4ce200,0x2000000,0x0,0x0,0x0,0x20140,0x0,0xe000,0xe000,0x2000000,0xb9f30d45,0x28920d40,0x40000,0x20140,0x0,0x20140,0x20020140,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x20,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x28920d40,0x0,0x0,0x28920d40,0x0,0x0,0x0,0x0,0x8000800,0x100000,0x0,0x0,0x0,0x8000800,0x8000000,0x28920d40,0x0,0x0,0x0,0x400,0x0,0x0,0xb9f30d45,0xb9f30d45,0xb9f30dc5,0x0,0x0,0x0,0x0,0x28920d40,0x0,0xb9f30d45,0x0,0x0,0x28920d40,0x28920d40,0x28920d40,0x28920d40,0x0,0x0,0x0,0x28920d40,0x0,0x0,};
  final private int[] jj_la1_2 = {0x0,0x0,0x20000,0x80000,0x20000,0x0,0x0,0x0,0x0,0x2100,0x0,0x0,0x100,0x0,0x0,0x0,0x0,0x0,0x100,0x100,0x0,0x0,0x40000,0x100000,0x8000,0x18029d1,0x18029d1,0x40000,0x0,0x0,0x0,0x22000,0x8000,0x40000,0x100,0x0,0x0,0x0,0x0,0x229d1,0x9d1,0x0,0x100,0x8000,0x0,0x100,0x40000,0x100000,0x100000,0x2000000,0x80000000,0x0,0x0,0x0,0x0,0x48000000,0x48000000,0x0,0x30600000,0x30600000,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x18009d1,0x1800000,0x1800000,0x9d1,0x0,0x0,0x800,0x800,0xd1,0x0,0x900,0x800,0x88000,0xd1,0x0,0x18009d1,0x40000,0x2000,0x8800,0x0,0x8000,0x8000,0x229d1,0x229d1,0x229d1,0x0,0x40000,0x100000,0x100000,0x9d1,0x0,0x229d1,0x0,0x0,0x9d1,0x18009d1,0x9d1,0x9d1,0x40000,0x100,0x100,0x18009d1,0x0,0x0,};
  final private int[] jj_la1_3 = {0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x1e,0x1e,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x6,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x1ffc000,0x1ffc000,0x0,0x0,0x1,0x100,0x200,0x80,0x0,0x0,0x0,0x0,0x0,0x3800,0x3800,0x18,0x18,0x460,0x460,0x18,0x1e,0x0,0x0,0x0,0x6,0x6,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x1e,0x0,0x0,0x0,0x0,0x0,0x0,0x6,0x6,0x6,0x0,0x0,0x1ffc006,0x1ffc006,0x6,0x0,0x6,0x0,0x0,0x6,0x1e,0x6,0x6,0x0,0x0,0x0,0x1e,0x0,0x0,};
  final private JJCalls[] jj_2_rtns = new JJCalls[3];
  private boolean jj_rescan = false;
  private int jj_gc = 0;

  public JavaParser(java.io.InputStream stream) {
    jj_input_stream = new ASCII_UCodeESC_CharStream(stream, 1, 1);
    token_source = new JavaParserTokenManager(jj_input_stream);
    token = new Token();
    jj_ntk = -1;
    jj_gen = 0;
    for (int i = 0; i < 111; i++) jj_la1[i] = -1;
    for (int i = 0; i < jj_2_rtns.length; i++) jj_2_rtns[i] = new JJCalls();
  }

  public void ReInit(java.io.InputStream stream) {
    jj_input_stream.ReInit(stream, 1, 1);
    token_source.ReInit(jj_input_stream);
    token = new Token();
    jj_ntk = -1;
    jj_gen = 0;
    for (int i = 0; i < 111; i++) jj_la1[i] = -1;
    for (int i = 0; i < jj_2_rtns.length; i++) jj_2_rtns[i] = new JJCalls();
  }

  public JavaParser(java.io.Reader stream) {
    jj_input_stream = new ASCII_UCodeESC_CharStream(stream, 1, 1);
    token_source = new JavaParserTokenManager(jj_input_stream);
    token = new Token();
    jj_ntk = -1;
    jj_gen = 0;
    for (int i = 0; i < 111; i++) jj_la1[i] = -1;
    for (int i = 0; i < jj_2_rtns.length; i++) jj_2_rtns[i] = new JJCalls();
  }

  public void ReInit(java.io.Reader stream) {
    jj_input_stream.ReInit(stream, 1, 1);
    token_source.ReInit(jj_input_stream);
    token = new Token();
    jj_ntk = -1;
    jj_gen = 0;
    for (int i = 0; i < 111; i++) jj_la1[i] = -1;
    for (int i = 0; i < jj_2_rtns.length; i++) jj_2_rtns[i] = new JJCalls();
  }

  public JavaParser(JavaParserTokenManager tm) {
    token_source = tm;
    token = new Token();
    jj_ntk = -1;
    jj_gen = 0;
    for (int i = 0; i < 111; i++) jj_la1[i] = -1;
    for (int i = 0; i < jj_2_rtns.length; i++) jj_2_rtns[i] = new JJCalls();
  }

  public void ReInit(JavaParserTokenManager tm) {
    token_source = tm;
    token = new Token();
    jj_ntk = -1;
    jj_gen = 0;
    for (int i = 0; i < 111; i++) jj_la1[i] = -1;
    for (int i = 0; i < jj_2_rtns.length; i++) jj_2_rtns[i] = new JJCalls();
  }

  final private Token jj_consume_token(int kind) throws ParseException {
    Token oldToken;
    if ((oldToken = token).next != null) token = token.next;
    else token = token.next = token_source.getNextToken();
    jj_ntk = -1;
    if (token.kind == kind) {
      jj_gen++;
      if (++jj_gc > 100) {
        jj_gc = 0;
        for (int i = 0; i < jj_2_rtns.length; i++) {
          JJCalls c = jj_2_rtns[i];
          while (c != null) {
            if (c.gen < jj_gen) c.first = null;
            c = c.next;
          }
        }
      }
      return token;
    }
    token = oldToken;
    jj_kind = kind;
    throw generateParseException();
  }

  final private boolean jj_scan_token(int kind) {
    if (jj_scanpos == jj_lastpos) {
      jj_la--;
      if (jj_scanpos.next == null) {
        jj_lastpos = jj_scanpos = jj_scanpos.next = token_source.getNextToken();
      } else {
        jj_lastpos = jj_scanpos = jj_scanpos.next;
      }
    } else {
      jj_scanpos = jj_scanpos.next;
    }
    if (jj_rescan) {
      int i = 0; Token tok = token;
      while (tok != null && tok != jj_scanpos) { i++; tok = tok.next; }
      if (tok != null) jj_add_error_token(kind, i);
    }
    return (jj_scanpos.kind != kind);
  }

  final public Token getNextToken() {
    if (token.next != null) token = token.next;
    else token = token.next = token_source.getNextToken();
    jj_ntk = -1;
    jj_gen++;
    return token;
  }

  final public Token getToken(int index) {
    Token t = lookingAhead ? jj_scanpos : token;
    for (int i = 0; i < index; i++) {
      if (t.next != null) t = t.next;
      else t = t.next = token_source.getNextToken();
    }
    return t;
  }

  final private int jj_ntk() {
    if ((jj_nt=token.next) == null)
      return (jj_ntk = (token.next=token_source.getNextToken()).kind);
    else
      return (jj_ntk = jj_nt.kind);
  }

  private java.util.Vector jj_expentries = new java.util.Vector();
  private int[] jj_expentry;
  private int jj_kind = -1;
  private int[] jj_lasttokens = new int[100];
  private int jj_endpos;

  private void jj_add_error_token(int kind, int pos) {
    if (pos >= 100) return;
    if (pos == jj_endpos + 1) {
      jj_lasttokens[jj_endpos++] = kind;
    } else if (jj_endpos != 0) {
      jj_expentry = new int[jj_endpos];
      for (int i = 0; i < jj_endpos; i++) {
        jj_expentry[i] = jj_lasttokens[i];
      }
      boolean exists = false;
      for (java.util.Enumeration enum = jj_expentries.elements(); enum.hasMoreElements();) {
        int[] oldentry = (int[])(enum.nextElement());
        if (oldentry.length == jj_expentry.length) {
          exists = true;
          for (int i = 0; i < jj_expentry.length; i++) {
            if (oldentry[i] != jj_expentry[i]) {
              exists = false;
              break;
            }
          }
          if (exists) break;
        }
      }
      if (!exists) jj_expentries.addElement(jj_expentry);
      if (pos != 0) jj_lasttokens[(jj_endpos = pos) - 1] = kind;
    }
  }

  final public ParseException generateParseException() {
    jj_expentries.removeAllElements();
    boolean[] la1tokens = new boolean[122];
    for (int i = 0; i < 122; i++) {
      la1tokens[i] = false;
    }
    if (jj_kind >= 0) {
      la1tokens[jj_kind] = true;
      jj_kind = -1;
    }
    for (int i = 0; i < 111; i++) {
      if (jj_la1[i] == jj_gen) {
        for (int j = 0; j < 32; j++) {
          if ((jj_la1_0[i] & (1<<j)) != 0) {
            la1tokens[j] = true;
          }
          if ((jj_la1_1[i] & (1<<j)) != 0) {
            la1tokens[32+j] = true;
          }
          if ((jj_la1_2[i] & (1<<j)) != 0) {
            la1tokens[64+j] = true;
          }
          if ((jj_la1_3[i] & (1<<j)) != 0) {
            la1tokens[96+j] = true;
          }
        }
      }
    }
    for (int i = 0; i < 122; i++) {
      if (la1tokens[i]) {
        jj_expentry = new int[1];
        jj_expentry[0] = i;
        jj_expentries.addElement(jj_expentry);
      }
    }
    jj_endpos = 0;
    jj_rescan_token();
    jj_add_error_token(0, 0);
    int[][] exptokseq = new int[jj_expentries.size()][];
    for (int i = 0; i < jj_expentries.size(); i++) {
      exptokseq[i] = (int[])jj_expentries.elementAt(i);
    }
    return new ParseException(token, exptokseq, tokenImage);
  }

  final public void enable_tracing() {
  }

  final public void disable_tracing() {
  }

  final private void jj_rescan_token() {
    jj_rescan = true;
    for (int i = 0; i < 3; i++) {
      JJCalls p = jj_2_rtns[i];
      do {
        if (p.gen > jj_gen) {
          jj_la = p.arg; jj_lastpos = jj_scanpos = p.first;
          switch (i) {
            case 0: jj_3_1(); break;
            case 1: jj_3_2(); break;
            case 2: jj_3_3(); break;
          }
        }
        p = p.next;
      } while (p != null);
    }
    jj_rescan = false;
  }

  final private void jj_save(int index, int xla) {
    JJCalls p = jj_2_rtns[index];
    while (p.gen > jj_gen) {
      if (p.next == null) { p = p.next = new JJCalls(); break; }
      p = p.next;
    }
    p.gen = jj_gen + xla - jj_la; p.first = token; p.arg = xla;
  }

  static final class JJCalls {
    int gen;
    Token first;
    int arg;
    JJCalls next;
  }

}
