/*
 * CodeAidPopup.java - part of the CodeAid plugin.
 * Copyright (C) 1999 Jason Ginchereau
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


package codeaid.popup;

import java.awt.Component;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import javax.swing.JPopupMenu;

import javax.swing.event.PopupMenuEvent;
import javax.swing.event.PopupMenuListener;

import org.gjt.sp.jedit.EditAction;
import org.gjt.sp.jedit.View;
import org.gjt.sp.jedit.gui.InputHandler;


public abstract class CodeAidPopup
    extends JPopupMenu
    implements PopupMenuListener, MouseListener
{
    protected String    typedText;
    protected HintPopup hint;


    protected CodeAidPopup() {
        addPopupMenuListener(this);
        typedText = "";
    }


    public void setStartingText(String text) {
        typedText = text;
    }


    public void setHint(HintPopup hint) {
        this.hint = hint;
    }


    protected KeyHandler getKeyHandler(InputHandler ih) {
        return new KeyHandler(ih);
    }


    public void setVisible(boolean vis) {
        super.setVisible(vis);
        if (vis) {
            updateTypedText();
        }
        else {
            if (hint != null) { hint.setVisible(false); }
        }
    }


    protected abstract void updateTypedText();


    // PopupMenuListener interface:
    public void popupMenuWillBecomeVisible(PopupMenuEvent evt) {
        View v = EditAction.getView((Component) evt.getSource());
        if (v != null) {
            InputHandler ih = v.getInputHandler();
            if (!(ih instanceof KeyHandler)) {
                KeyHandler kh = getKeyHandler(ih);
                this.addKeyListener(kh);
                v.setInputHandler(kh);
            }
        }
    }


    public void popupMenuWillBecomeInvisible(PopupMenuEvent evt) {
        View v = EditAction.getView((Component) evt.getSource());
        if (v != null) {
            InputHandler ih = v.getInputHandler();
            if (ih instanceof KeyHandler) {
                v.setInputHandler(((KeyHandler)ih).getOriginalInputHandler());
                this.removeKeyListener(ih);
            }
        }
        typedText = "";
    }


    public void popupMenuCanceled(PopupMenuEvent evt) { }


    // MouseListener interface:
    public void mouseClicked(MouseEvent evt) {}
    public void mousePressed(MouseEvent evt) {}
    public void mouseReleased(MouseEvent evt) {}
    public void mouseEntered(MouseEvent evt) {}
    public void mouseExited(MouseEvent evt) {}


    protected class KeyHandler extends InputHandlerAdapter {
        public KeyHandler(InputHandler ih) {
            super(ih);
        }


        public void keyTyped(KeyEvent evt) {
            super.keyTyped(evt);
            char c = evt.getKeyChar();
            if(c != KeyEvent.CHAR_UNDEFINED && c != '\b')
            typedText += c;
        }


        public void keyPressed(KeyEvent evt) {
            switch(evt.getKeyCode()) {
            case KeyEvent.VK_BACK_SPACE:
                super.keyPressed(evt);
                if (typedText.length() > 0) {
                    typedText = typedText.substring(0, typedText.length() - 1);
                } else {
                    setVisible(false);
                }
                break;

            case KeyEvent.VK_ESCAPE:
                setVisible(false);
                break;

            case KeyEvent.VK_LEFT:
            case KeyEvent.VK_RIGHT:
            case KeyEvent.VK_HOME:
            case KeyEvent.VK_END:
            case KeyEvent.VK_DELETE:
                setVisible(false);
                // fall through

            default:
                super.keyPressed(evt);
            }
        }
    }
}
