/*
 * HintPopup.java - part of the CodeAid plugin.
 * Copyright (C) 1999 Jason Ginchereau
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


package codeaid.popup;

import java.awt.Container;
import java.awt.Point;
import java.awt.Rectangle;
import javax.swing.*;
import org.gjt.sp.jedit.textarea.JEditTextArea;


public class HintPopup extends JToolTip {
    /**
     * Between JLayeredPane.DEFAULT_LAYER and JLayeredPane.PALETTE_LAYER.
    **/
    public static final Integer HINT_LAYER =
        new Integer((JLayeredPane.DEFAULT_LAYER.intValue() +
                     JLayeredPane.PALETTE_LAYER.intValue()) / 2);

    private JEditTextArea textArea;
    private boolean vis;


    public HintPopup(JEditTextArea ta) {
        this.textArea = ta;
        this.vis = false;
    }


    public void setHint(String hint) {
        setTipText(hint);
        setSize(getPreferredSize());
    }


    private void setTALocation(int x, int y) {
        Point p = SwingUtilities.convertPoint(textArea, x, y,
            textArea.getRootPane().getLayeredPane());
        super.setLocation(p.x, p.y);
    }


    public void setTextLocation(int line, int col) {
        setTALocation(textArea.offsetToX(line, col), textArea.lineToY(line-1)+3);
    }


    public boolean isVisible() {
        return vis;
    }


    public void setVisible(boolean vis) {
        if (vis) {
            _show();
        } else {
            _hide();
        }
    }


    private void _show() {
        if (!vis) {
            textArea.getRootPane().getLayeredPane().add(this, HINT_LAYER, 0);
            vis = true;
        }
    }


    private void _hide() {
        if (vis) {
            Container parent = getParent();
            Rectangle r = getBounds();
            if (parent != null) {
                parent.remove(this);
                parent.repaint(r.x,r.y,r.width,r.height);
            }
            vis = false;
        }
    }
}
