/*
 * MemberListCellRenderer.java - part of the CodeAid plugin.
 * Copyright (C) 1999 Jason Ginchereau
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


package codeaid.popup;

import codeaid.info.FieldInfo;
import codeaid.info.MemberInfo;
import codeaid.info.MethodInfo;

import java.awt.*;
import javax.swing.*;


public class MemberListCellRenderer extends JPanel implements ListCellRenderer {
    private JLabel typeLabel;
    private JLabel nameLabel;
    private JLabel paramLabel;
    private JLabel closeLabel;


    public MemberListCellRenderer(Object[] listData) {
        typeLabel = new JLabel();
        nameLabel = new JLabel();
        paramLabel = new JLabel();
        closeLabel = new JLabel();

        JPanel panel2 = new JPanel();
        panel2.setOpaque(false);
        panel2.setLayout(new BorderLayout());
        panel2.add(BorderLayout.WEST, nameLabel);
        panel2.add(BorderLayout.CENTER, paramLabel);

        JPanel panel3 = new JPanel();
        panel3.setOpaque(false);
        panel3.setLayout(new BorderLayout());
        panel3.add(BorderLayout.WEST, panel2);
        panel3.add(BorderLayout.CENTER, closeLabel);

        setLayout(new BorderLayout());
        add(BorderLayout.WEST, typeLabel);
        add(BorderLayout.CENTER, panel3);
    }


    public int getNameOffset() {
        return typeLabel.getPreferredSize().width;
    }


    public void setFont(Font f) {
        super.setFont(f);
        if(nameLabel != null) {
            Font smallerFont = new Font("SansSerif", Font.PLAIN,
                                        Math.max(f.getSize() - 4, 10));
            typeLabel.setFont(smallerFont);
            nameLabel.setFont(f);
            paramLabel.setFont(smallerFont);
            closeLabel.setFont(f);

            typeLabel.setPreferredSize(null);
            typeLabel.setText("ObjectObject");
            typeLabel.setPreferredSize(typeLabel.getPreferredSize());
        }
    }


    public Component getListCellRendererComponent(JList list, Object value,
                                                  int index, boolean isSelected,
                                                  boolean cellHasFocus) {
        if (value instanceof FieldInfo) {
            typeLabel.setText(lastName(((MemberInfo) value).getType()));
            nameLabel.setText(((MemberInfo) value).getName());
            paramLabel.setText("");
            closeLabel.setText("");
        }
        else if (value instanceof MethodInfo) {
            typeLabel.setText(lastName(((MemberInfo) value).getType()));
            nameLabel.setText(((MemberInfo) value).getName() + "(");

            String params = "";
            String[] paramNames = ((MethodInfo) value).getParameterNames();
            String[] paramTypes = ((MethodInfo) value).getParameterTypes();
            for (int i = 0; i < paramTypes.length; i++) {
                if (i > 0) { params += ", "; }
                params += lastName(paramTypes[i]);
                if (paramNames != null) { params += " " + paramNames[i]; }
            }
            paramLabel.setText(params);

            closeLabel.setText(")");
        }
        else {
            typeLabel.setText("");
            nameLabel.setText(value.toString());
            paramLabel.setText("");
            closeLabel.setText("");
        }

        Color background = (isSelected ? list.getSelectionBackground()
                                       : list.getBackground());
        Color foreground = (isSelected ? list.getSelectionForeground()
                                       : list.getForeground());
        setBackground(background);
        typeLabel.setForeground(foreground);
        nameLabel.setForeground(foreground);
        paramLabel.setForeground(foreground);
        closeLabel.setForeground(foreground);

        return this;
    }

    private static String lastName(String name) {
        int i = name.lastIndexOf('.');
        return (i >= 0) ? name.substring(i + 1) : name;
    }
}
