/*
 * CDBEntry.java
 * Copyright (c) 2000 Andre Kaplan
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


package codeaid.tools;

import java.io.File;
import java.util.Enumeration;
import java.util.Hashtable;

import org.gjt.sp.jedit.jEdit;
import org.gjt.sp.util.Log;


public class CDBEntry {
    public String name = "";
    public String lib  = "";
    public boolean privateInfo  = false;
    public boolean fullComments = false;
    public boolean valid        = true;


    public CDBEntry() {}


    public CDBEntry(String name) {
        this.name = name;
    }


    public CDBEntry(String name, String lib, boolean privateInfo,
            boolean fullComments) {
        this.name = name;
        this.lib  = lib;
        this.privateInfo  = privateInfo;
        this.fullComments = fullComments;
    }


    public static void load() {
        // load libs from properties:
        int count = 0;
        do {
            String cdbName = jEdit.getProperty("codeaid.lib.name." + count);
            if (cdbName == null || cdbName.length() == 0) break;
            String cdbLib = jEdit.getProperty("codeaid.lib.lib." + count);
            if (cdbLib == null || cdbLib.length() == 0) break;
            boolean privateInfo = jEdit.getBooleanProperty(
                "codeaid.lib.privateinfo." + count, false);
            boolean fullComments = jEdit.getBooleanProperty(
                "codeaid.lib.fullcomments." + count, false);
            cdbEntries.put(cdbName,
                    new CDBEntry(cdbName, cdbLib, privateInfo, privateInfo)
            );
            count++;
        } while (true);

        // Load libs from dir
        File cdbDir = new File(jEdit.getSettingsDirectory() + File.separator
            + "codeaid");
        if(!cdbDir.exists()) {
            cdbDir.mkdir();
        } else {
            String[] fileList = cdbDir.list();
            for(int i = 0; i < fileList.length; i++) {
                if(!fileList[i].endsWith(".cdb")) continue;

                if (cdbEntries.get(fileList[i]) == null) {
                    Log.log(Log.MESSAGE, CDBEntry.class,
                        "Add CDB entry: " + fileList[i]
                    );
                    cdbEntries.put(fileList[i], new CDBEntry(fileList[i]));
                }
            }
        }
    }


    public static void save() {
        int count = 0;
        for (Enumeration e = cdbEntries.elements(); e.hasMoreElements(); ) {
            CDBEntry l = (CDBEntry) e.nextElement();
            if (l == null) continue;
            if (l.name == null || l.name.length() == 0) continue;
            // if (l.lib == null || l.lib.length() == 0) continue;
            jEdit.setProperty("codeaid.lib.name." + count, l.name);
            jEdit.setProperty("codeaid.lib.lib." + count, l.lib);
            jEdit.setBooleanProperty("codeaid.lib.privateinfo." + count,
                l.privateInfo);
            jEdit.setBooleanProperty("codeaid.lib.fullcomments." + count,
                l.fullComments);
            count++;
        }
        jEdit.unsetProperty("codeaid.lib.name." + count);
        jEdit.unsetProperty("codeaid.lib.lib." + count);
        jEdit.unsetProperty("codeaid.lib.privateinfo." + count);
        jEdit.unsetProperty("codeaid.lib.fullcomments." + count);
    }


    public static Object add(CDBEntry entry) {
        if (entry == null) { return null; }
        if (entry.name == null || entry.name.length() == 0) { return null; }

        return cdbEntries.put(entry.name, entry);
    }


    public static Object remove(String entryName) {
        if (entryName == null || entryName.length() == 0) { return null; }

        return cdbEntries.remove(entryName);
    }


    public static Enumeration entries() {
        return cdbEntries.elements();
    }


    private static Hashtable cdbEntries = new Hashtable();
}
