/*
 * ClassReader.java
 * Copyright (C) 1999-2001 Jason Ginchereau, Andre Kaplan
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


package codeaid.tools;

import codeaid.info.ClassInfo;
import codeaid.info.ClassTable;

import java.io.IOException;
import java.io.ByteArrayOutputStream;
import java.io.InputStream;


public class ClassReader extends ClassInfoReader {
    private Loader loader;


    ClassReader(String filename, ClassTable ct) {
        super(filename, ct, 1);
        this.loader = new Loader();
    }


    private static class Loader extends ClassLoader {
        Class loadClass(String name, byte[] bytes) {
            Class c = findLoadedClass(name);
            if (c != null) { return c; }
            return defineClass(name, bytes, 0, bytes.length);
        }


        protected Class loadClass(String name, boolean resolve) {
            Class clazz = findLoadedClass(name);
            if (clazz != null) {
                if (resolve) {
                    resolveClass(clazz);
                }
            }
            return clazz;
        }
    }


    protected void process(InputStream in, String path, ClassTable ct,
        int numPasses)
            throws IOException
    {
        if(path.endsWith(".class")) {
            String name = path.substring(0, path.length() - 6);
            name = name.replace('/', '.');
            while(name.startsWith(".")) { name = name.substring(1); }

            try {
                ByteArrayOutputStream b = new ByteArrayOutputStream();
                int n;
                byte[] buf = new byte[2048];
                while ((n = in.available()) > 0) {
                    n = in.read(buf, 0, buf.length);
                    b.write(buf, 0, n);
                }
                Class c = loader.loadClass(name, b.toByteArray());
                if (c != null) { ct.put(new ClassInfo(c)); }
            } catch(RuntimeException rex) {
                throw rex;
            } catch(Exception ex) {
                throw new IOException(ex.toString());
            }
        }
    }
}

