/*
 * DBWriter.java
 * Copyright (C) 1999-2001 Jason Ginchereau, Andre Kaplan
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


package codeaid.tools;

import codeaid.info.ClassTable;

import java.io.IOException;
import java.io.File;
import java.io.FileOutputStream;

import java.util.zip.ZipOutputStream;

import org.gjt.sp.util.Log;


public class DBWriter implements Runnable {
    private ClassTable ct;
    private File dbFile;


    DBWriter(String filename, ClassTable ct) {
        this.dbFile = new File(filename);
        this.ct = ct;

        if (dbFile.exists()) {
            Log.log(Log.WARNING, this, "\"" + dbFile + "\"" + " already exists."
                + "Overwriting it");
        }
        if (!dbFile.getName().toLowerCase().endsWith(".cdb")) {
            throw new IllegalArgumentException(dbFile.getName() + " does not "
                + "end with \".cdb\"."
            );
        }
    }


    public void run() {
        try {
            ct.write(new ZipOutputStream(new FileOutputStream(dbFile)));
        } catch(IOException ioex) {
            ioex.printStackTrace(System.err);
        }
    }
}

