/*
 * SourceReader.java
 * Copyright (C) 1999-2001 Jason Ginchereau, Andre Kaplan
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


package codeaid.tools;

import codeaid.info.ClassInfo;
import codeaid.info.ClassTable;
import codeaid.parser.JavaParser;

import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;

import org.gjt.sp.util.Log;


public class SourceReader extends ClassInfoReader {
    private JavaParser parser;


    SourceReader(String filename, ClassTable ct, boolean oneLineComments,
            boolean privateInfo) {
        super(filename, ct, 2);

        parser = new JavaParser();
        parser.generateClassInfo = true;
        parser.generatePrivateClassInfo = privateInfo;
        parser.oneLineComments = oneLineComments;
    }


    protected void process(InputStream in, String path, ClassTable ct, int n)
            throws IOException
    {
        if (path.endsWith(".java")) {
            if (n == 0) {
                ct = ct.getParent();
                path = path.substring(0, path.length() - 5);
                path = path.replace('\\', '/');
                path = path.replace('/', '.');
                int i = path.lastIndexOf('.');
                if (i >= 0) {
                    ct.put(new ClassInfo(path.substring(i+1), path.substring(0, i)));
                } else {
                    ct.put(new ClassInfo(path, null));
                }
            }
            else if (n == 1) {
                parser.init(
                    new InputStreamReader(in) {
                        // cancel close
                        public void close() throws IOException { }
                    },
                    path
                );
                parser.classTable = ct;
                parser.run();
                Log.log(Log.MESSAGE, this, path);
            }
        }
    }
}

